 
/*******************************************************************************

  
  Copyright(c) 1999 - 2003 Intel Corporation. All rights reserved.
  
  This program is free software; you can redistribute it and/or modify it 
  under the terms of the GNU General Public License as published by the Free 
  Software Foundation; either version 2 of the License, or (at your option) 
  any later version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
  more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  The full GNU General Public License is included in this distribution in file called LICENSE.
  
  Contact Information:
  Linux NICS <linux.nics@intel.com>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
*******************************************************************************/

/**********************************************************************
*                                                                     *
* INTEL CORPORATION                                                   *
*                                                                     *
* This software is supplied under the terms of the license included   *
* above.  All use of this driver must be in accordance with the terms *
* of that license.                                                    *
*                                                                     *
* Module Name:  e100_main.c                                           *
*                                                                     *
* Abstract:     Functions for the driver entry points like load,      *
*               unload, open and close. All board specific calls made *
*               by the network interface section of the driver.       *
*                                                                     *
* Environment:  This file is intended to be specific to the Linux     *
*               operating system.                                     *
*                                                                     *
**********************************************************************/

/* Change Log
 * 
 * 2.2.21	02/11/03
 * o Removed marketing brand strings. Instead, Using generic string 
 *   "Intel(R) PRO/100 Network Connection" for all adapters.
 * o Implemented ethtool -S option
 * o Strip /proc/net/PRO_LAN_Adapters files for kernel driver
 * o Bug fix: Read wrong byte in EEPROM when offset is odd number
 * o Bug fix: PHY loopback test fails on ICH devices
 * o Bug fix: System panic on e100_close when repeating Hot Remove and 
 *   Add in a team
 * o Bug fix: Linux Bonding driver claims adapter's link loss because of
 *   not updating last_rx field
 * o Bug fix: e100 does not check validity of MAC address
 * o New feature: added ICH5 support
 * 
 * 2.1.27	11/20/02
 *   o Bug fix: Device command timeout due to SMBus processing during init
 *   o Bug fix: Not setting/clearing I (Interrupt) bit in tcb correctly
 *   o Bug fix: Not using EEPROM WoL setting as default in ethtool
 *   o Bug fix: Not able to set autoneg on using ethtool when interface down
 *   o Bug fix: Not able to change speed/duplex using ethtool/mii
 *     when interface up
 *   o Bug fix: Ethtool shows autoneg on when forced to 100/Full
 *   o Bug fix: Compiler error when CONFIG_PROC_FS not defined
 *   o Bug fix: 2.5.44 e100 doesn't load with preemptive kernel enabled
 *     (sleep while holding spinlock)
 *   o Bug fix: 2.1.24-k1 doesn't display complete statistics
 *   o Bug fix: System panic due to NULL watchdog timer dereference during
 *     ifconfig down, rmmod and insmod
 *
 * 2.1.24       10/7/02
 **********************************************************************
Changelog: June 29,2004: The file is changed by, Deyu Liu,
to port the e100 adapter to RTLinux-GPL
Computer Science Department, The Florida State University,USA
<bullleo2003@yahoo.com>

CHANGELOG:    November      25,    2004    Sergio    Prez     Alcaiz
(serpeal@upvnet.upv.es):

   o  Bug fix: the  use of the function schedule_timeout(HZ  / 50 + 1)
   can cause an  exception in the  RTLinux context. It  should only be
   used in the Linux context. It's  been replaced with usleep().   
   o Bug   fix: the use  of the  function set_current_state()  is only
   needed in the  Linux driver.  Using it in  the  RTLinux context may
   cause Linux to be hanged.

CHANGELOG:     December    1,      2004     Sergio     Prez   Alcaiz
(serpeal@upvnet.upv.es):

  o Driver modified to provide the O_NOBLOCK access mode (non-blocking 
  access). 
  o Driver  modified  to  support  the  zero  copy  technique  or  not, 
  depending on the options set in the reddopts.h.
  o Driver modified to support the FAKE architecture  described in the
  reddoopts.h

**********************************************************************
*/
 
#include <linux/config.h>
#include <net/checksum.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include "drivers/rtl_e100main_drv.h"
#include "drivers/rtl_e100_ucode.h"
#include "drivers/rtl_e100_config.h"
#include "drivers/rtl_e100_phy.h"


MODULE_AUTHOR("Intel Corporation, <linux.nics@intel.com>");
MODULE_DESCRIPTION("Intel(R) PRO/100 Network Driver");
MODULE_LICENSE("GPL");


static int rt_e100_interrupted = 0, rt_e100_writting = 0;

extern void e100_force_speed_duplex_to_phy(void);

static void e100_get_speed_duplex_caps(void);

#define E100_EEPROM_MAGIC 0x1234


extern void e100_config_wol(void);

/* Global Data structures and variables */
char e100_copyright[] __devinitdata = "Copyright (c) 2003 Intel Corporation";
char e100_driver_version[]="2.2.21-k1";
const char *e100_full_driver_name = "Intel(R) PRO/100 Network Driver";
char e100_short_driver_name[] = "e100";
static int e100nics = 0;

#ifdef CONFIG_PM
/*
struct notifier_block e100_notifier_reboot = {
        .notifier_call  = e100_notify_reboot,
        .next           = NULL,
        .priority       = 0
	};*/
#endif

/*********************************************************************/
/*! This is a GCC extension to ANSI C.
 *  See the item "Labeled Elements in Initializers" in the section
 *  "Extensions to the C Language Family" of the GCC documentation.
 *********************************************************************/
#define E100_PARAM_INIT { [0 ... E100_MAX_NIC] = -1 }

/* All parameters are treated the same, as an integer array of values.
 * This macro just reduces the need to repeat the same declaration code
 * over and over (plus this helps to avoid typo bugs).
 */
#define E100_PARAM(X, S)                                        \
        static const int X[E100_MAX_NIC + 1] = E100_PARAM_INIT; \
        MODULE_PARM(X, "1-" __MODULE_STRING(E100_MAX_NIC) "i"); \
        MODULE_PARM_DESC(X, S);

/* ====================================================================== */
static int e100_open(struct pci_dev *);
static int e100_found1(struct pci_dev *pcid);
static inline void e100_tx_skb_free(tcb_t *tcb);
static inline struct rx_list_elem *e100_get_rx_struct(void);
static unsigned char e100_init(void);
void e100_isolate_driver(void);
static unsigned int e100intr(unsigned int, struct pt_regs *);
static void rt_e100_print_brd_conf(void);

static int e100_pci_setup(struct pci_dev *);
static u8 e100_sw_init(void);
static void e100_tco_workaround(void);
static unsigned char e100_alloc_space(void);
static void e100_dealloc_space(void);
static int e100_alloc_tcb_pool(void);
static void e100_setup_tcb_pool(tcb_t *, unsigned int);
static void e100_free_tcb_pool(void);
static int e100_alloc_rfd_pool(void);
static void e100_free_rfd_pool(void);

static void e100_rd_eaddr(void);
static void e100_rd_pwa_no(void);
extern u16 e100_eeprom_read( u16);
extern void e100_eeprom_write_block( u16, u16 *, u16);
extern u16 e100_eeprom_size(void);
u16 e100_eeprom_calculate_chksum(void);

static unsigned char e100_clr_cntrs(void);
static unsigned char e100_load_microcode(void);
static unsigned char e100_hw_init(void);
static unsigned char e100_setup_iaaddr( u8 *);

static void e100_start_ru(void);
static void e100_check_options(int board);
static void e100_set_int_option(int *, int, int, int, int, char *);
static void e100_set_bool_option(int, u32, int,
				 char *);
unsigned char e100_wait_exec_cmplx( u32, u8, u8);
void e100_exec_cmplx( u32, u8);

static inline tcb_t *e100_prepare_xmit_buff(const char *buffer, size_t size);

/***************************************************************************************/
/* This function is called in the cleanup_module function, and it must do other stuff  */
/* not done in the cleanup_module function in order to "close" the driver. In some     */
/* implementations this function could be empty.                                       */
/***************************************************************************************/
static void other_stuff_closing(void){
}


/***************************************************************************************/
/* This function is used to send a packet. It writes into the internal buffers of the  */
/* card.                                                                               */
/***************************************************************************************/
static int rt_eth_send_packet(const char *buffer, size_t size)
{

  rt_e100_writting = 1;

  e100_prepare_xmit_buff(buffer,size);

  private_data.cur_tx++;

  rt_e100_writting = 0;

  return size;
}

/***************************************************************************************/
/* This function does internal initialisations of the card. It writes into internal    */
/* registers and checks the card capabilities.                                         */
/***************************************************************************************/
int start_up_device(struct pci_dev *dev){
  int rc=0;
  rc=e100_found1(dev);

return rc;

}
/**
 * e100_get_rx_struct - retrieve cell to hold skb buff from the pool
 * @private_data: atapter's private data struct
 *
 * Returns the new cell to hold sk_buff or %NULL.
 */
static inline struct rx_list_elem *e100_get_rx_struct()
{
	struct rx_list_elem *rx_struct = NULL;

	if (!list_empty(&(private_data.rx_struct_pool))) {
		rx_struct = list_entry(private_data.rx_struct_pool.next,
				       struct rx_list_elem, list_elem);
		list_del(&(rx_struct->list_elem));
	}

	return rx_struct;
}

/**
 * e100_alloc_skb - allocate an skb for the adapter
 * @private_data: atapter's private data struct
 *
 * Allocates skb with enough room for rfd, and data, and reserve non-data space.
 * Returns the new cell with sk_buff or %NULL.
 */
static inline struct rx_list_elem *
e100_alloc_skb(void)
{
	unsigned char *new_skb;
	u32 skb_size = sizeof (rfd_t);
	struct rx_list_elem *rx_struct;

	new_skb = (unsigned char *) rtl_malloc(skb_size);
	if (new_skb) {
		/* The IP data should be 
		   DWORD aligned. since the ethernet header is 14 bytes long, 
		   we need to reserve 2 extra bytes so that the TCP/IP headers
		   will be DWORD aligned. modified
		skb_reserve(new_skb, 2);*/
		if ((rx_struct = e100_get_rx_struct()) == NULL)
			goto err;
		rx_struct->skb = new_skb;
		rx_struct->dma_addr = pci_map_single(private_data.pdev, new_skb, sizeof (rfd_t),PCI_DMA_FROMDEVICE);
		if (!rx_struct->dma_addr)
			goto err;
		/* reserve the header*/
		rx_struct->skb += private_data.rfd_size;
		return rx_struct;
	} else {
		return NULL;
	}

err:

	return NULL;
}

/**
 * e100_add_skb_to_end - add an skb to the end of our rfd list
 * @private_data: atapter's private data struct
 * @rx_struct: rx_list_elem with the new skb
 *
 * Adds a newly allocated skb to the end of our rfd list.
 */
inline void
e100_add_skb_to_end(struct rx_list_elem *rx_struct)
{
	rfd_t *rfdn;		/* The new rfd */
	rfd_t *rfd;		/* The old rfd */
	struct rx_list_elem *rx_struct_last;

	rfdn = RFD_POINTER(rx_struct->skb, private_data);
	rfdn->rfd_header.cb_status = 0;
	/*indicate the last RFD*/
	rfdn->rfd_header.cb_cmd = __constant_cpu_to_le16(RFD_EL_BIT);
	rfdn->rfd_act_cnt = 0;
	rfdn->rfd_sz = __constant_cpu_to_le16(RFD_DATA_SIZE);
	/*for the last RFD, wrap around*/
	if (private_data.skb_req==1)
	  put_unaligned(cpu_to_le32(((struct rx_list_elem *)(private_data.active_rx_list.next))->dma_addr),
			((u32 *) (&(rfdn->rfd_header.cb_lnk_ptr))));


	pci_dma_sync_single(private_data.pdev, rx_struct->dma_addr, private_data.rfd_size,
			    PCI_DMA_TODEVICE);

	if (!list_empty(&(private_data.active_rx_list))) {
		rx_struct_last = list_entry(private_data.active_rx_list.prev,
					    struct rx_list_elem, list_elem);
		rfd = RFD_POINTER(rx_struct_last->skb, private_data);
		pci_dma_sync_single(private_data.pdev, rx_struct_last->dma_addr,
				    4, PCI_DMA_FROMDEVICE);
		put_unaligned(cpu_to_le32(rx_struct->dma_addr),
			      ((u32 *) (&(rfd->rfd_header.cb_lnk_ptr))));

		pci_dma_sync_single(private_data.pdev, rx_struct_last->dma_addr,
				    8, PCI_DMA_TODEVICE);
		rfd->rfd_header.cb_cmd &=
			__constant_cpu_to_le16((u16) ~RFD_EL_BIT);

		pci_dma_sync_single(private_data.pdev, rx_struct_last->dma_addr,
				    4, PCI_DMA_TODEVICE);
	}

	list_add_tail(&(rx_struct->list_elem), &(private_data.active_rx_list));
}

static inline void
e100_alloc_skbs(void)
{
	for (; private_data.skb_req > 0; private_data.skb_req--) {
		struct rx_list_elem *rx_struct;

		if ((rx_struct = e100_alloc_skb()) == NULL)
			return;

		e100_add_skb_to_end( rx_struct);
	}
}

void e100_tx_srv(void);
u32 e100_rx_srv(void);

void e100_clear_pools(void);
static void e100_clear_structs(struct pci_dev *);


E100_PARAM(TxDescriptors, "Number of transmit descriptors");
E100_PARAM(RxDescriptors, "Number of receive descriptors");
E100_PARAM(XsumRX, "Disable or enable Receive Checksum offload");
E100_PARAM(e100_speed_duplex, "Speed and Duplex settings");
E100_PARAM(ucode, "Disable or enable microcode loading");
E100_PARAM(ber, "Value for the BER correction algorithm");
E100_PARAM(flow_control, "Disable or enable Ethernet PAUSE frames processing");
E100_PARAM(IntDelay, "Value for CPU saver's interrupt delay");
E100_PARAM(BundleSmallFr, "Disable or enable interrupt bundling of small frames");
E100_PARAM(BundleMax, "Maximum number for CPU saver's packet bundling");
E100_PARAM(IFS, "Disable or enable the adaptive IFS algorithm");

/**
 * e100_exec_cmd - issue a comand
 * @private_data: atapter's private data struct
 * @scb_cmd_low: the command that is to be issued
 *
 * This general routine will issue a command to the e100.
 */
static inline void
e100_exec_cmd(u8 cmd_low)
{
	writeb(cmd_low, &(private_data.scb->scb_cmd_low));
	readw(&(private_data.scb->scb_status));	
}

/**
 * e100_wait_scb - wait for SCB to clear
 * @private_data: atapter's private data struct
 *
 * This routine checks to see if the e100 has accepted a command.
 * It does so by checking the command field in the SCB, which will
 * be zeroed by the e100 upon accepting a command.  The loop waits
 * for up to 1 millisecond for command acceptance.
 *
 * Returns:
 *      true if the SCB cleared within 1 millisecond.
 *      false if it didn't clear within 1 millisecond
 */
unsigned char
e100_wait_scb()
{
	int i;

	/* loop on the scb for a few times */
	for (i = 0; i < 100; i++) {
		if (!readb(&private_data.scb->scb_cmd_low))
			return true;
		cpu_relax();
	}

	/* it didn't work. do it the slow way using udelay()s */
	for (i = 0; i < E100_MAX_SCB_WAIT; i++) {
		if (!readb(&private_data.scb->scb_cmd_low))
			return true;
		cpu_relax();
		udelay(1);
	}

	return false;
}

/**
 * e100_wait_exec_simple - issue a command
 * @private_data: atapter's private data struct
 * @scb_cmd_low: the command that is to be issued
 *
 * This general routine will issue a command to the e100 after waiting for
 * the previous command to finish.
 *
 * Returns:
 *      true if the command was issued to the chip successfully
 *      false if the command was not issued to the chip
 */
inline unsigned char
e100_wait_exec_simple(u8 scb_cmd_low)
{
  /*wait one scb command to be ACKed*/
	if (!e100_wait_scb()) {
		rtl_printf(KERN_DEBUG "e100: e100_wait_exec_simple: failed\n");
#ifdef E100_CU_DEBUG		
		rtl_printf(KERN_ERR "e100: Last command (%x/%x) "
			"timeout\n",  
			private_data.last_cmd, private_data.last_sub_cmd);
		rtl_printf(KERN_ERR "e100: Current simple command (%x) "
			"can't be executed\n", scb_cmd_low);
#endif		
		return false;
	}
	e100_exec_cmd( scb_cmd_low);
#ifdef E100_CU_DEBUG	
	private_data.last_cmd = scb_cmd_low;
	private_data.last_sub_cmd = 0;
#endif	
	return true;
}

void
e100_exec_cmplx(u32 phys_addr, u8 cmd)
{
	writel(phys_addr, &(private_data.scb->scb_gen_ptr));
	readw(&(private_data.scb->scb_status));	/* flushes last write, read-safe */
	e100_exec_cmd( cmd);
}

unsigned char
e100_wait_exec_cmplx(u32 phys_addr, u8 cmd, u8 sub_cmd)
{
	if (!e100_wait_scb()) {
#ifdef E100_CU_DEBUG		
		rtl_printf(KERN_ERR "e100: Last command (%x/%x) "
			"timeout\n",  
			private_data.last_cmd, private_data.last_sub_cmd);
		rtl_printf(KERN_ERR "e100: Current complex command "
			"(%x/%x) can't be executed\n", 
			 cmd, sub_cmd);
#endif		
		return false;
	}
	e100_exec_cmplx( phys_addr, cmd);
#ifdef E100_CU_DEBUG	
	private_data.last_cmd = cmd;
	private_data.last_sub_cmd = sub_cmd;
#endif	
	return true;
}

inline u8
e100_wait_cus_idle(void)
{
	int i;

	/* loop on the scb for a few times */
	for (i = 0; i < 100; i++) {
		if (((readw(&(private_data.scb->scb_status)) & SCB_CUS_MASK) !=
		     SCB_CUS_ACTIVE)) {
			return true;
		}
		cpu_relax();
	}

	for (i = 0; i < E100_MAX_CU_IDLE_WAIT; i++) {
		if (((readw(&(private_data.scb->scb_status)) & SCB_CUS_MASK) !=
		     SCB_CUS_ACTIVE)) {
			return true;
		}
		cpu_relax();
		udelay(1);
	}

	return false;
}

/**
 * e100_disable_clear_intr - disable and clear/ack interrupts
 * @private_data: atapter's private data struct
 *
 * This routine disables interrupts at the hardware, by setting
 * the M (mask) bit in the adapter's CSR SCB command word.
 * It also clear/ack interrupts.
 */
static inline void
e100_disable_clear_intr(void)
{
	u16 intr_status;
	/* Disable interrupts on our PCI board by setting the mask bit */
	writeb(SCB_INT_MASK, &private_data.scb->scb_cmd_hi);
	intr_status = readw(&private_data.scb->scb_status);
	/* ack and clear intrs */
	writew(intr_status, &private_data.scb->scb_status);
	readw(&private_data.scb->scb_status);
}

/**
 * e100_set_intr_mask - set interrupts
 * @private_data: atapter's private data struct
 *
 * This routine sets interrupts at the hardware, by resetting
 * the M (mask) bit in the adapter's CSR SCB command word
 */
static inline void
e100_set_intr_mask(void)
{
	writeb(private_data.intr_mask, &private_data.scb->scb_cmd_hi);
	readw(&(private_data.scb->scb_status)); /* flushes last write, read-safe */
}

static inline void
e100_trigger_SWI(void)
{
	/* Trigger interrupt on our PCI board by asserting SWI bit */
	writeb(SCB_SOFT_INT, &private_data.scb->scb_cmd_hi);
	readw(&(private_data.scb->scb_status));	
}

static int e100_found1(struct pci_dev *pcid)
{
	static int first_time = true;
	int rc = 0;
	u16 cal_checksum, read_checksum;

	if (first_time) {
		first_time = false;
        	rtl_printf(KERN_NOTICE "%s - version %s\n",
	               e100_full_driver_name, e100_driver_version);
		rtl_printf(KERN_NOTICE "%s\n", e100_copyright);
		rtl_printf(KERN_NOTICE "\n");
	}

	private_data.pdev = pcid;
	private_data.last_tcb = NULL;
	/*alloc bd_dma_able_t to private str*/
	if ((rc = e100_alloc_space()) != 0) {
		goto err_dev;
	}

	private_data.flags = 0;
	private_data.ifs_state = 0;
	private_data.ifs_value = 0;
	private_data.scb = 0;

	/*enable and reserve pci resources and scb*/
	if ((rc = e100_pci_setup(pcid)) != 0) {
	  rtl_printf("e100_pci_setup is not successful\n");
	  goto err_dealloc;
	}

	if (private_data.rev_id == 0xff)
		private_data.rev_id = 1;

	if ((u8) private_data.rev_id >= D101A4_REV_ID)
	  {
	    private_data.flags |= IS_BACHELOR;
	  }

	if ((u8) private_data.rev_id >= D102_REV_ID) {
		private_data.flags |= USE_IPCB;
		private_data.rfd_size = 32;
	} else {
		private_data.rfd_size = 16;
	}
	/*check command line options*/
	e100_check_options(e100nics);



	if (!e100_init()) {
		rtl_printf(KERN_ERR "e100: Failed to initialize, instance #%d\n",
		       e100nics);
		rc = -ENODEV;
		goto err_pci;
	}




	/* Check if checksum is valid */
	cal_checksum = e100_eeprom_calculate_chksum();
	read_checksum = e100_eeprom_read( (private_data.eeprom_size - 1));
	if (cal_checksum != read_checksum) {
                rtl_printf(KERN_ERR "e100: Corrupted EEPROM on instance #%d\n",
		       e100nics);
                rc = -ENODEV;
                goto err_pci;
	}

	e100nics++;

	e100_get_speed_duplex_caps();

        memcpy(private_data.ifname, "rtle100", IFNAMSIZ);
        private_data.ifname[IFNAMSIZ-1] = 0;	

	rtl_printf(KERN_NOTICE
	       "e100: %s: %s\n", 
	       private_data.ifname, "Intel(R) PRO/100 Network Connection");
	rt_e100_print_brd_conf();

	private_data.wolsupported = 0;
	private_data.wolopts = 0;
	
	/* Check if WoL is enabled on EEPROM */
	if (e100_eeprom_read( EEPROM_ID_WORD) & BIT_5) {
		/* Magic Packet WoL is enabled on device by default */
		/* if EEPROM WoL bit is TRUE                        */
		private_data.wolsupported = WAKE_MAGIC;
		private_data.wolopts = WAKE_MAGIC;
		if (private_data.rev_id >= D101A4_REV_ID)
			private_data.wolsupported = WAKE_PHY | WAKE_MAGIC;
		if (private_data.rev_id >= D101MA_REV_ID)
			private_data.wolsupported |= WAKE_UCAST | WAKE_ARP;
		rtl_printf(KERN_NOTICE "***EEPROM WoL bit is TRUE\n");
	}

	rtl_printf("antes del open\n");

	e100_open(pcid);

	rtl_printf("despues del open\n");

	goto out;

err_pci:
	iounmap(private_data.scb);
	pci_release_regions(pcid);
	pci_disable_device(pcid);
err_dealloc:
	e100_dealloc_space();
err_dev:
	pci_set_drvdata(pcid, NULL);

out:
	return rc;
}

/**
 * e100_clear_structs - free resources
 * @dev: adapter's net_device struct
 *
 * Free all device specific structs, unmap i/o address, etc.
 */
static void
e100_clear_structs(struct pci_dev *dev)
{
 
	iounmap(private_data.scb);
	/**Releases the PCI I/O and memory resources reserved by a
pci_request_region() call**/
	pci_release_regions(dev);
	/**disabling PCI bus-mastering**/
	pci_disable_device(dev);
	/*deallocate bd_dma_able_t structure**/
	e100_dealloc_space();
	pci_set_drvdata(dev, NULL);

}

static void eth_remove_one(struct pci_dev *pcid)
{
  e100_clear_structs(pcid);
  --e100nics;
}

/**
 * e100_check_options - check command line options
 * @board: board number
 * @private_data: atapter's private data struct
 *
 * This routine does range checking on command-line options
 */
void
e100_check_options(int board)
{
	if (board >= E100_MAX_NIC) {
		rtl_printf(KERN_NOTICE 
		       "e100: No configuration available for board #%d\n",
		       board);
		rtl_printf(KERN_NOTICE "e100: Using defaults for all values\n");
		board = E100_MAX_NIC;
	}

	e100_set_int_option(&(private_data.params.TxDescriptors), TxDescriptors[board],
			    E100_MIN_TCB, E100_MAX_TCB, E100_DEFAULT_TCB,
			    "TxDescriptor count");

	e100_set_int_option(&(private_data.params.RxDescriptors), RxDescriptors[board],
			    E100_MIN_RFD, E100_MAX_RFD, E100_DEFAULT_RFD,
			    "RxDescriptor count");

	e100_set_int_option(&(private_data.params.e100_speed_duplex),
			    e100_speed_duplex[board], 0, 4,
			    E100_DEFAULT_SPEED_DUPLEX, "speed/duplex mode");

	e100_set_int_option(&(private_data.params.ber), ber[board], 0, ZLOCK_MAX_ERRORS,
			    E100_DEFAULT_BER, "Bit Error Rate count");

	e100_set_bool_option( XsumRX[board], PRM_XSUMRX, E100_DEFAULT_XSUM,
			     "XsumRX value");

	/* Default ucode value depended on controller revision */
	if (private_data.rev_id >= D101MA_REV_ID) {
		e100_set_bool_option( ucode[board], PRM_UCODE,
				     E100_DEFAULT_UCODE, "ucode value");
	} else {
		e100_set_bool_option( ucode[board], PRM_UCODE, false,
				     "ucode value");
	}

	e100_set_bool_option( flow_control[board], PRM_FC, E100_DEFAULT_FC,
			     "flow control value");

	e100_set_bool_option( IFS[board], PRM_IFS, E100_DEFAULT_IFS,
			     "IFS value");

	e100_set_bool_option( BundleSmallFr[board], PRM_BUNDLE_SMALL,
			     E100_DEFAULT_BUNDLE_SMALL_FR,
			     "CPU saver bundle small frames value");

	e100_set_int_option(&(private_data.params.IntDelay), IntDelay[board], 0x0,
			    0xFFFF, E100_DEFAULT_CPUSAVER_INTERRUPT_DELAY,
			    "CPU saver interrupt delay value");

	e100_set_int_option(&(private_data.params.BundleMax), BundleMax[board], 0x1,
			    0xFFFF, E100_DEFAULT_CPUSAVER_BUNDLE_MAX,
			    "CPU saver bundle max value");

}

/**
 * e100_set_int_option - check and set an integer option
 * @option: a pointer to the relevant option field
 * @val: the value specified
 * @min: the minimum valid value
 * @max: the maximum valid value
 * @default_val: the default value
 * @name: the name of the option
 *
 * This routine does range checking on a command-line option.
 * If the option's value is '-1' use the specified default.
 * Otherwise, if the value is invalid, change it to the default.
 */
void 
e100_set_int_option(int *option, int val, int min, int max, int default_val,
		    char *name)
{
	if (val == -1) {	/* no value specified. use default */
		*option = default_val;

	} else if ((val < min) || (val > max)) {
		rtl_printf(KERN_NOTICE
		       "e100: Invalid %s specified (%i). "
		       "Valid range is %i-%i\n",
		       name, val, min, max);
		rtl_printf(KERN_NOTICE "e100: Using default %s of %i\n", name,
		       default_val);
		*option = default_val;
	} else {
		rtl_printf(KERN_INFO "e100: Using specified %s of %i\n", name, val);
		*option = val;
	}
}

/**
 * e100_set_bool_option - check and set a boolean option
 * @private_data: atapter's private data struct
 * @val: the value specified
 * @mask: the mask for the relevant option
 * @default_val: the default value
 * @name: the name of the option
 *
 * This routine checks a boolean command-line option.
 * If the option's value is '-1' use the specified default.
 * Otherwise, if the value is invalid (not 0 or 1), 
 * change it to the default.
 */
void
e100_set_bool_option(int val, u32 mask,
		     int default_val, char *name)
{
	if (val == -1) {
		if (default_val)
			private_data.params.b_params |= mask;

	} else if ((val != true) && (val != false)) {
		rtl_printf(KERN_NOTICE
		       "e100: Invalid %s specified (%i). "
		       "Valid values are %i/%i\n",
		       name, val, false, true);
		rtl_printf(KERN_NOTICE "e100: Using default %s of %i\n", name,
		       default_val);

		if (default_val)
			private_data.params.b_params |= mask;
	} else {
		rtl_printf(KERN_INFO "e100: Using specified %s of %i\n", name, val);
		if (val)
			private_data.params.b_params |= mask;
	}
}

static int
e100_open(struct pci_dev *pdev)
{
	int retval,rc = 0;
	u16 int_reg;
	/*print the params*/
	/*read the control register from phy*/
	e100_mdi_read( MII_BMSR,private_data.phy_addr, &int_reg);
	e100_mdi_read( MII_BMSR,private_data.phy_addr, &int_reg);

	/* setup the tcb pool */
	if (!e100_alloc_tcb_pool()) {
		rc = -ENOMEM;
		rtl_printf(KERN_ERR "***e100_alloc_tcb_pool failed\n");
		goto err_exit;
	}
	private_data.last_tcb = NULL;

	private_data.tcb_pool.head = 1;/*modified from 0 to 1*/
	private_data.tcb_pool.tail = 1;	

	e100_setup_tcb_pool((tcb_t *) private_data.tcb_pool.data,
			    private_data.params.TxDescriptors);

	if (!e100_alloc_rfd_pool()) {
	  rtl_printf(KERN_ERR "***e100_alloc_rfd_pool failed\n");
		rc = -ENOMEM;
		goto err_exit;
	}

	/*for receiving pool*/
	private_data.rvhead = (struct rx_list_elem *)(private_data.active_rx_list.next);
	private_data.rvprev = (struct rx_list_elem *)(private_data.active_rx_list.prev);

	if (!e100_wait_exec_cmplx( 0, SCB_CUC_LOAD_BASE, 0)) {
		rc = -EAGAIN;
		goto err_exit;
	}

	if (!e100_wait_exec_cmplx( 0, SCB_RUC_LOAD_BASE, 0)) {
		rc = -EAGAIN;
		goto err_exit;
	}
     
	e100_start_ru();

	/* Use the now-standard shared IRQ implementation. */
	/* IRQ is got from pci_enable_device() */
	if ((retval = rtl_request_irq(pdev->irq, &e100intr))) {
	  rtl_printf(KERN_ERR "%s: Could not reserve IRQ %d\n", pdev->name,
		 pdev->irq);
	  goto err_exit;
	}else
	  private_data.must_free_irq = 1;

	/* allow interrupts*/
	private_data.intr_mask = 0;
	e100_set_intr_mask();

	e100_force_config();

        e100_mdi_read( MII_BMSR,private_data.phy_addr, &int_reg);
        e100_mdi_read( MII_BMSR,private_data.phy_addr, &int_reg);

        e100_mdi_read( MII_ADVERTISE,private_data.phy_addr, &int_reg);
        e100_mdi_read( MII_ADVERTISE,private_data.phy_addr, &int_reg);

	private_data.next_cu_cmd = START_WAIT;
	goto exit;

err_exit:
	e100_clear_pools();
exit:
	return rc;
}

static int
eth_close(struct pci_dev *dev)
{

	e100_disable_clear_intr();
	if(private_data.must_free_irq){
	  rtl_free_irq(dev->irq);
	  private_data.must_free_irq = 0;
	}

	private_data.intr_mask = SCB_INT_MASK;

	private_data.cur_line_speed = 0;
	private_data.cur_dplx_mode = 0;
	e100_clear_pools();

	return 0;
}

/**
 * e100init - initialize the adapter
 * @private_data: atapter's private data struct
 *
 * This routine is called when this driver is loaded. This is the initialization
 * routine which allocates memory, configures the adapter and determines the
 * system resources.
 *
 * Returns:
 *      true: if successful
 *      false: otherwise
 */
static unsigned char
e100_init()
{
	u32 st_timeout = 0;
	u32 st_result = 0;

	e100_sw_init();

	if (!e100_selftest( &st_timeout, &st_result)) {
        	if (st_timeout) {
			rtl_printf(KERN_ERR "e100: selftest timeout\n");
		} else {
			rtl_printf(KERN_ERR "e100: selftest failed. Results: %x\n",
					st_result);
		}
		return false;
	}

	/* read the MAC address from the eprom */
	e100_rd_eaddr();

	if (!is_valid_ether_addr(private_data.dev_addr)) {
		rtl_printf(KERN_ERR "e100: Invalid Ethernet address\n");
		return false;
	}

	/* read NIC's part number */
	e100_rd_pwa_no();

	if (!e100_hw_init()) {
		rtl_printf(KERN_ERR "e100: hw init failed\n");
		return false;
	}

	/* Interrupts are enabled after device reset */
	e100_disable_clear_intr();

	return true;
}

/**
 * e100_sw_init - initialize software structs
 * @private_data: atapter's private data struct
 * 
 * This routine initializes all software structures. Sets up the
 * circular structures for the RFD's & TCB's. Allocates the per board
 * structure for storing adapter information. The CSR is also memory 
 * mapped in this routine.
 *
 * Returns :
 *      true: if S/W was successfully initialized
 *      false: otherwise
 */
static unsigned char
e100_sw_init()
{
  /*for the first TX command*/
	private_data.next_cu_cmd = START_WAIT;	// init the next cu state

	/* 
	 * Set the value for # of good xmits per underrun. the value assigned
	 * here is an intelligent  suggested default. Nothing magical about it.
	 */
	private_data.tx_per_underrun = DEFAULT_TX_PER_UNDERRUN;

	/* get the default transmit threshold value */
	private_data.tx_thld = TX_THRSHLD;

	/* get the EPROM size */
	private_data.eeprom_size = e100_eeprom_size();

	/* Initialize our spinlocks */
	spin_lock_init(&(private_data.bd_lock));
	spin_lock_init(&(private_data.bd_non_tx_lock));
	spin_lock_init(&(private_data.config_lock));
	spin_lock_init(&(private_data.mdi_access_lock));

	return 1;
}

static void 
e100_tco_workaround()
{
	int i;

	/* Do software reset */
	e100_sw_reset( PORT_SOFTWARE_RESET);

	/* Do a dummy LOAD CU BASE command. */
	/* This gets us out of pre-driver to post-driver. */
	e100_exec_cmplx( 0, SCB_CUC_LOAD_BASE);

	/* Wait 20 msec for reset to take effect */
//BUG:	set_current_state(TASK_UNINTERRUPTIBLE);

	rtl_printf("usleep 1\n");
	usleep(20000);
//BUG:	schedule_timeout(HZ / 50 + 1);

	/* disable interrupts since they are enabled */
	/* after device reset                        */
	e100_disable_clear_intr();

	/* Wait for command to be cleared up to 1 sec */
	for (i=0; i<100; i++) {
		if (!readb(&private_data.scb->scb_cmd_low))
			break;
//BUG:		set_current_state(TASK_UNINTERRUPTIBLE);
		rtl_printf("usleep 2\n");
		usleep(1000000);
//BUG:		schedule_timeout(HZ / 100 + 1);
	}

	/* Wait for TCO request bit in PMDR register to be clear */
	for (i=0; i<50; i++) {
		if (!(readb(&private_data.scb->scb_ext.d101m_scb.scb_pmdr) & BIT_1))
			break;
//BUG:		set_current_state(TASK_UNINTERRUPTIBLE);
		rtl_printf("usleep 3\n");
		usleep(1000000);
//BUG:		schedule_timeout(HZ / 100 + 1);
	}
}

/**
 * e100_hw_init - initialized tthe hardware
 * @private_data: atapter's private data struct
 *
 * This routine performs a reset on the adapter, and configures the adapter.
 * This includes configuring the 82557 LAN controller, validating and setting
 * the node address, detecting and configuring the Phy chip on the adapter,
 * and initializing all of the on chip counters.
 *
 * Returns:
 *      true - If the adapter was initialized
 *      false - If the adapter failed initialization
 */
unsigned char 
e100_hw_init()
{
  /**set params in rtl_e100main_phy.c***/

	if (!e100_phy_init())
		return false;

	e100_sw_reset(PORT_SELECTIVE_RESET);


	/* Only 82559 or above needs TCO workaround */
	if (private_data.rev_id >= D101MA_REV_ID)
		{
		  e100_tco_workaround();
		  rtl_printf(KERN_ERR "***e100_tco_workaround Finished\n");
		}

	/* Load the CU BASE (set to 0, because we use linear mode) */
	if (!e100_wait_exec_cmplx( 0, SCB_CUC_LOAD_BASE, 0))
		return false;


	if (!e100_wait_exec_cmplx( 0, SCB_RUC_LOAD_BASE, 0))
		return false;

	/* Load interrupt microcode  */
	if (e100_load_microcode()) {
		private_data.flags |= DF_UCODE_LOADED;
	}

	e100_config_init();

	if (!e100_config()) {
		return false;
	}

	if (!e100_setup_iaaddr( private_data.dev_addr))
		return false;

	/* Clear the internal counters */
	if (!e100_clr_cntrs())
		return false;

	/* Change for 82558 enhancement */
	/* If 82558/9 and if the user has enabled flow control, set up the
	 * Flow Control Reg. in the CSR */

	if ((private_data.flags & IS_BACHELOR)
	    && (private_data.params.b_params & PRM_FC)) {
		writeb(DFLT_FC_THLD, &private_data.scb->scb_ext.d101_scb.scb_fc_thld);
		writeb(DFLT_FC_CMD,
		       &private_data.scb->scb_ext.d101_scb.scb_fc_xon_xoff);
		//rtl_printf(KERN_ERR "***FC done\n");
	}
	return true;
}

/**
 * e100_setup_tcb_pool - setup TCB circular list
 *and initialize each TCB 
 * @head: Pointer to head of the allocated TCBs
 * @qlen: Number of elements in the queue
 * @private_data: atapter's private data struct
 * 
 * This routine arranges the contigiously allocated TCB's in a circular list.
 * Also does the one time initialization of the TCBs.
 */
static void
e100_setup_tcb_pool(tcb_t *head, unsigned int qlen)
{
	int ele_no;
	tcb_t *pcurr_tcb;	/* point to current tcb */
	u32 next_phys;		/* the next phys addr */
	u16 txcommand = CB_S_BIT | CB_TX_SF_BIT;
	unsigned char *skb;

	private_data.tx_count = 0;
	if (private_data.flags & USE_IPCB) {
		txcommand |= CB_IPCB_TRANSMIT | CB_CID_DEFAULT;
	} else if (private_data.flags & IS_BACHELOR) {
		txcommand |= CB_TRANSMIT | CB_CID_DEFAULT;
	} else {
		txcommand |= CB_TRANSMIT;
	}

	for (ele_no = 0, next_phys = private_data.tcb_phys, pcurr_tcb = head;
	     ele_no < qlen; ele_no++, pcurr_tcb++) {

		/* set the phys addr for this TCB, next_phys has not incr. yet */
		pcurr_tcb->tcb_phys = next_phys;
		next_phys += sizeof (tcb_t);

		/* set up the circular tcb list */
		if (ele_no == (qlen - 1))
			pcurr_tcb->tcb_hdr.cb_lnk_ptr =
				cpu_to_le32(private_data.tcb_phys);
		else
			pcurr_tcb->tcb_hdr.cb_lnk_ptr = cpu_to_le32(next_phys);

		pcurr_tcb->tcb_hdr.cb_status = 0;
		pcurr_tcb->tcb_hdr.cb_cmd = cpu_to_le16(txcommand);
		pcurr_tcb->tcb_cnt = 0;	
		pcurr_tcb->tcb_thrshld = private_data.tx_thld;	
		pcurr_tcb->tcb_tbd_num = 1;

		if (private_data.flags & IS_BACHELOR) {
			pcurr_tcb->tcb_tbd_ptr =
				__constant_cpu_to_le32(0xFFFFFFFF);
		} else {
			pcurr_tcb->tcb_tbd_ptr =
				cpu_to_le32(pcurr_tcb->tcb_phys + 0x10);
		}

		if (private_data.flags & IS_BACHELOR) {
			pcurr_tcb->tcb_tbd_expand_ptr =
				cpu_to_le32(pcurr_tcb->tcb_phys + 0x20);
		} else {
			pcurr_tcb->tcb_tbd_expand_ptr =
				cpu_to_le32(pcurr_tcb->tcb_phys + 0x10);
		}
		pcurr_tcb->tcb_tbd_dflt_ptr = pcurr_tcb->tcb_tbd_ptr;

		if (private_data.flags & USE_IPCB) {
			pcurr_tcb->tbd_ptr = &(pcurr_tcb->tcbu.tbd_array[1]);
			pcurr_tcb->tcbu.ipcb.ip_activation_high =
				IPCB_IP_ACTIVATION_DEFAULT;
			pcurr_tcb->tcbu.ipcb.vlan = 0;
		} else {
			pcurr_tcb->tbd_ptr = &(pcurr_tcb->tcbu.tbd_array[0]);
		}

		/*alloc skb to the curr tcb*/
		skb = rtl_malloc(PKT_BUF_SZ);
		pcurr_tcb->tcb_skb = skb;
		if (skb == NULL)
		  break;                  /* Bad news!  */
		(pcurr_tcb->tbd_ptr)->tbd_buf_addr =
		  cpu_to_le32(pci_map_single(private_data.pdev, skb, PKT_BUF_SZ,
					     PCI_DMA_TODEVICE));
	} /* for */
	wmb();

	if (ele_no < qlen) {

	  rtl_printf(KERN_EMERG "ele_no = %d: no enough memory for tx ring\n",ele_no);

	  rtl_printf(KERN_EMERG "\nPlease unload the module and try again\n\n");
	}

}

/***************************************************************************/
/***************************************************************************/
/*       Memory Management Routines                                        */
/***************************************************************************/

/**
 * e100_alloc_space - allocate private driver data
 * @private_data: atapter's private data struct
 *
 * This routine allocates memory for the driver. Memory allocated is for the
 * selftest and statistics structures.
 *
 * Returns:
 *      0: if the operation was successful
 *      %-ENOMEM: if memory allocation failed
 */
unsigned char e100_alloc_space()
{
	unsigned long off;

	/* allocate all the dma-able structures in one call:
	 * selftest results, adapter stats, and non-tx cb commands */
	if (!(private_data.dma_able =
	      pci_alloc_consistent(private_data.pdev, sizeof (bd_dma_able_t),
				   &(private_data.dma_able_phys)))) {
		goto err;
	}

	/* now assign the various pointers into the struct we've just allocated */
	off = offsetof(bd_dma_able_t, selftest);

	private_data.selftest = (self_test_t *) (private_data.dma_able + off);
	private_data.selftest_phys = private_data.dma_able_phys + off;

	off = offsetof(bd_dma_able_t, stats_counters);

	private_data.stats_counters = (max_counters_t *) (private_data.dma_able + off);
	private_data.stat_cnt_phys = private_data.dma_able_phys + off;

	return 0;

err:
	rtl_printf(KERN_ERR
	       "e100: Failed to allocate memory\n");
	return -ENOMEM;
}

/**
 * e100_alloc_tcb_pool - allocate TCB circular list
 * @private_data: atapter's private data struct
 *
 * This routine allocates memory for the circular list of transmit descriptors.
 *
 * Returns:
 *       0: if allocation has failed.
 *       1: Otherwise. 
 */
int
e100_alloc_tcb_pool()
{
	int stcb = sizeof (tcb_t) * private_data.params.TxDescriptors;

	/* allocate space for the TCBs */
	if (!(private_data.tcb_pool.data =
	      pci_alloc_consistent(private_data.pdev, stcb, &private_data.tcb_phys)))
		return 0;

	memset(private_data.tcb_pool.data, 0x00, stcb);

	return 1;
}

void
e100_free_tcb_pool()
{
  int i;
  tcb_t * tcb;
  /*skb has to be free*/
  for (i = 0; i < private_data.params.TxDescriptors; i++) 
    {
      tcb = private_data.tcb_pool.data;
      tcb += i;
      mb();
      e100_tx_skb_free(tcb);
      mb();  
    }
	pci_free_consistent(private_data.pdev,
			    sizeof (tcb_t) * private_data.params.TxDescriptors,
			    private_data.tcb_pool.data, private_data.tcb_phys);
	private_data.tcb_phys = 0;
}

static void
e100_dealloc_space()
{
	if (private_data.dma_able) {
		pci_free_consistent(private_data.pdev, sizeof (bd_dma_able_t),
				    private_data.dma_able, private_data.dma_able_phys);
	}

	private_data.selftest_phys = 0;
	private_data.stat_cnt_phys = 0;
	private_data.dma_able_phys = 0;
	private_data.dma_able = 0;
}

static void
e100_free_rfd_pool()
{
	struct rx_list_elem *rx_struct;

	while (!list_empty(&(private_data.active_rx_list))) {

		rx_struct = list_entry(private_data.active_rx_list.next,
				       struct rx_list_elem, list_elem);
		list_del(&(rx_struct->list_elem));
		pci_unmap_single(private_data.pdev, rx_struct->dma_addr,
				 sizeof (rfd_t), PCI_DMA_TODEVICE);
		/*add the header, or it would crashe the system*/
                rx_struct->skb -= private_data.rfd_size;
		rtl_free(rx_struct->skb);
		kfree(rx_struct);
	}

	while (!list_empty(&(private_data.rx_struct_pool))) {
		rx_struct = list_entry(private_data.rx_struct_pool.next,
				       struct rx_list_elem, list_elem);
		list_del(&(rx_struct->list_elem));
		kfree(rx_struct);
	}
}

/**
 * e100_alloc_rfd_pool - allocate RFDs
 * @private_data: atapter's private data struct
 *
 * Allocates initial pool of skb which holds both rfd and data,
 * and return a pointer to the head of the list
 */
static int
e100_alloc_rfd_pool()
{
	struct rx_list_elem *rx_struct;
	int i;

	INIT_LIST_HEAD(&(private_data.active_rx_list));
	INIT_LIST_HEAD(&(private_data.rx_struct_pool));
	private_data.skb_req = private_data.params.RxDescriptors;
	for (i = 0; i < private_data.skb_req; i++) {
		rx_struct = kmalloc(sizeof (struct rx_list_elem), GFP_ATOMIC);
		list_add(&(rx_struct->list_elem), &(private_data.rx_struct_pool));
	}
	e100_alloc_skbs();
	return !list_empty(&(private_data.active_rx_list));

}

void
e100_clear_pools()
{
	private_data.last_tcb = NULL;
	e100_free_rfd_pool();
	e100_free_tcb_pool();
}

/**
 * e100intr - interrupt handler
 * @irq: the IRQ number
 * @dev_inst: the net_device struct
 * @regs: registers (unused)
 *
 * This routine is the ISR for the e100 board. It services
 * the RX & TX queues & starts the RU if it has stopped due
 * to no resources.
 */
unsigned int
e100intr(unsigned int irq, struct pt_regs *regs)
{

	u16 intr_status;
	
	cli();
	
	inside_the_interrupt_handler = 1;
	rt_e100_interrupted++;


	if(rt_e100_writting & rt_e100_interrupted)
	  rtl_printf("I've been interrupted %d times\n",rt_e100_interrupted);
	
	
	intr_status = readw(&private_data.scb->scb_status);

	/* If not my interrupt, just return */
	if (!(intr_status & SCB_STATUS_ACK_MASK) || (intr_status == 0xffff)) {
		return 1;
	}

	/* disable and ack intr */
	e100_disable_clear_intr();

	/* do recv work if any */

	if (intr_status &
	    (SCB_STATUS_ACK_FR | SCB_STATUS_ACK_RNR )) 
		private_data.drv_stats.rx_intr_pkts += e100_rx_srv();

	/* clean up after tx'ed packets */
	if (intr_status & (SCB_STATUS_ACK_CNA | SCB_STATUS_ACK_CX))
		e100_tx_srv();

	e100_set_intr_mask();

	/* We must be sure that we're out of the interrupt handler before
	   cleanup_modules */
	/* is executed. If cleanup_modules is being executed, we don't have
	   to enable the */
	/* irq. If enabled, then the system could crash.
	 */
	if(!trying_to_close)
	  rtl_hard_enable_irq(private_data.pdev->irq);

	rt_e100_interrupted--;
	sti();
	return (inside_the_interrupt_handler = 0);

}

/**
 * e100_tx_skb_free - free TX skbs resources
 * @private_data: atapter's private data struct
 * @tcb: associated tcb of the freed skb
 *
 * This routine frees resources of TX skbs.
 */
static inline void
e100_tx_skb_free(tcb_t *tcb)
{
	if (tcb->tcb_skb) {

		tbd_t *tbd_arr = tcb->tbd_ptr;
		
		pci_unmap_single(private_data.pdev,
				 le32_to_cpu(tbd_arr->tbd_buf_addr),
				 le16_to_cpu(tbd_arr->tbd_buf_cnt),
				 PCI_DMA_TODEVICE);
		rtl_free(tcb->tcb_skb);
		tcb->tcb_skb = NULL;
	}
}

/**
 * e100_tx_srv - service TX queues
 * @private_data: atapter's private data struct
 *
 * This routine services the TX queues. It reclaims the TCB's & TBD's & other
 * resources used during the transmit of this buffer. It is called from the ISR.
 * We don't need a tx_lock since we always access buffers which were already
 * prepared.
 */
void
e100_tx_srv()
{
	tcb_t *tcb;
	int i;

	/*non-tx initiate CNA intr before e100_setup_tcb_pool() called*/
	if (private_data.last_tcb !=NULL)
	/* go over at most TxDescriptors buffers */
	for (i = 0; i < private_data.params.TxDescriptors; i++) {
		tcb = private_data.tcb_pool.data;
		tcb += private_data.tcb_pool.head;

		rmb();

		/* if the buffer at 'head' is not complete, break */
		if (!(tcb->tcb_hdr.cb_status &
		      __constant_cpu_to_le16(CB_STATUS_COMPLETE)))
		{
		  break;
		}

		/* service next buffer, clear the out of resource condition */
		tcb->tcb_hdr.cb_status = 0;

		/* if we've caught up with 'tail', break */
		if (private_data.tcb_pool.head == private_data.tcb_pool.tail) {
			break;
		}

		private_data.tcb_pool.head = NEXT_TCB_TOUSE(private_data.tcb_pool.head);
		mb();
	}

}

/**
 * e100_rx_srv - service RX queue
 * @private_data: atapter's private data struct
 * @max_number_of_rfds: max number of RFDs to process
 * @rx_congestion: flag pointer, to inform the calling function of congestion.
 */
u32
e100_rx_srv()
{
	rfd_t *rfdhead, *rfdprev;	  /* new rfd, received rfd */
	int i;
	u16 rfd_statushead;
	unsigned char *skbhead, *skbprev;
	unsigned int data_sz;
	struct rx_list_elem *rx_structhead, *rx_structprev;
	u32 rfd_cnt = 0;

	for (i = 0; i < private_data.params.RxDescriptors; i++) {
	  rx_structhead =private_data.rvhead;
	  skbhead = rx_structhead->skb;
	  
	  rfdhead = RFD_POINTER(skbhead, private_data);	/* locate RFD within skb */
	  
	  // sync only the RFD header
	  pci_dma_sync_single(private_data.pdev, rx_structhead->dma_addr,
			      private_data.rfd_size, PCI_DMA_FROMDEVICE);
	  rfd_statushead = le16_to_cpu(rfdhead->rfd_header.cb_status);	/* get RFD's status */

	  if (!(rfd_statushead & RFD_STATUS_COMPLETE))	/* does not contains data yet - exit */
	    {
	      /*clear the EL bit to the prev RFD*/
	      rx_structprev =private_data.rvprev;
	      skbprev = rx_structprev->skb;
	      rfdprev = RFD_POINTER(skbprev, private_data);
	      rfdprev->rfd_header.cb_status = 0;
	      rfdprev->rfd_header.cb_cmd &= __constant_cpu_to_le16((u16) ~RFD_EL_BIT);
	      rfdprev->rfd_act_cnt = 0;
	      rfdprev->rfd_sz = __constant_cpu_to_le16(RFD_DATA_SIZE);
	      mb();
	      
	      private_data.rvprev = PREV_RV_USED(private_data.rvhead);
	      /*set the EL bit to the last RFD */
	      rx_structprev =private_data.rvprev;
 	      skbprev = rx_structprev->skb;
	      rfdprev = RFD_POINTER(skbprev, private_data);
	      rfdprev->rfd_header.cb_status = 0;
	      rfdprev->rfd_header.cb_cmd = __constant_cpu_to_le16(RFD_EL_BIT);
	      rfdprev->rfd_act_cnt = 0;
	      mb();	      
	      break;
	    }
	  
	  data_sz = min_t(u16, (le16_to_cpu(rfdhead->rfd_act_cnt) & 0x3fff),
			  (sizeof (rfd_t) - private_data.rfd_size));
	  
	  /* now sync all the data */
	  pci_dma_sync_single(private_data.pdev, rx_structhead->dma_addr,
			      (data_sz + private_data.rfd_size),
			      PCI_DMA_FROMDEVICE);

	  
	  /* set packet size, excluding checksum (2 last bytes) if it is present */
	  if ((private_data.flags & DF_CSUM_OFFLOAD)
	      && (private_data.rev_id < D102_REV_ID))
	    data_sz -= 2;

	  /** similar to netif_rx()**/
	  filter_incoming_packet(skbhead,data_sz);
	  mb();
	  /*claim the dirty RFD*/
	  rfdhead->rfd_header.cb_status = 0;
	  rfdhead->rfd_header.cb_cmd &= 
	    __constant_cpu_to_le16((u16) ~RFD_EL_BIT);
	  rfdhead->rfd_act_cnt = 0;
	  rfdhead->rfd_sz = __constant_cpu_to_le16(RFD_DATA_SIZE);
	  
	  private_data.rvhead = NEXT_RV_TOUSE(private_data.rvhead);
			  
	  mb();
	  rfd_cnt++;
	 }			/* end of rfd loop */

	/* restart the RU if it has stopped */
	if ((readw(&private_data.scb->scb_status) & SCB_RUS_MASK) != SCB_RUS_READY) {
	   rtl_printf(KERN_ERR"***Now e100_start_ru()\n");
	  e100_start_ru();
	}
	
	return rfd_cnt;
}

/**
 * e100_prepare_xmit_buff - prepare a buffer for transmission
 * @private_data: atapter's private data struct
 * @skb: skb to send
 *

 */
static inline tcb_t *
e100_prepare_xmit_buff(const char *buffer, size_t size)
{
  unsigned char *buff;
  tcb_t *tcb=NULL, *prev_tcb;


	tcb = private_data.tcb_pool.data;
	tcb += TCB_TO_USE(private_data.tcb_pool);

	if (private_data.flags & USE_IPCB) {
		tcb->tcbu.ipcb.ip_activation_high = IPCB_IP_ACTIVATION_DEFAULT;
		tcb->tcbu.ipcb.ip_schedule &= ~IPCB_TCP_PACKET;
		tcb->tcbu.ipcb.ip_schedule &= ~IPCB_TCPUDP_CHECKSUM_ENABLE;
	}
	tcb->tcb_hdr.cb_status = 0;
	tcb->tcb_thrshld = private_data.tx_thld;
	tcb->tcb_hdr.cb_cmd |= __constant_cpu_to_le16(CB_S_BIT);
	wmb();
	/* Set I (Interrupt) bit on every (TX_FRAME_CNT)th packet */
	if (!(++private_data.tx_count % TX_FRAME_CNT))
		tcb->tcb_hdr.cb_cmd |= __constant_cpu_to_le16(CB_I_BIT);
	else
		/* Clear I bit on other packets */
		tcb->tcb_hdr.cb_cmd &= ~__constant_cpu_to_le16(CB_I_BIT);
	wmb();

	buff=tcb->tcb_skb;

	if(buff)
          memcpy(buff, buffer, size);

	(tcb->tbd_ptr)->tbd_buf_cnt = cpu_to_le16(size);
	tcb->tcb_tbd_num = 1;
	tcb->tcb_tbd_ptr = tcb->tcb_tbd_dflt_ptr;
	wmb();
	/* clear the S-BIT on the previous tcb */
	prev_tcb = private_data.tcb_pool.data;
	prev_tcb += PREV_TCB_USED(private_data.tcb_pool);
	prev_tcb->tcb_hdr.cb_cmd &= __constant_cpu_to_le16((u16) ~CB_S_BIT);

	if (private_data.tcb_pool.head == NEXT_TCB_TOUSE(private_data.tcb_pool.tail))
	  {
	    /*no memory for TX*/
	    rtl_printf(KERN_ERR "\n*** no memory for TX***\n");
	    return NULL;
	  }
	private_data.tcb_pool.tail = NEXT_TCB_TOUSE(private_data.tcb_pool.tail);
	wmb();


	e100_start_cu( tcb);

	return tcb;
}

/* Changed for 82558 enhancement */
/**
 * e100_start_cu - start the adapter's CU
 * @private_data: atapter's private data struct
 * @tcb: TCB to be transmitted
 *
 * This routine issues a CU Start or CU Resume command to the 82558/9.
 * This routine was added because the prepare_ext_xmit_buff takes advantage
 * of the 82558/9's Dynamic TBD chaining feature and has to start the CU as
 * soon as the first TBD is ready. 
 *
 * e100_start_cu must be called while holding the tx_lock ! 
 */
u8
e100_start_cu(tcb_t *tcb)
{
  //unsigned long lock_flag;
	u8 ret = true;

	switch (private_data.next_cu_cmd) {
	case RESUME_NO_WAIT:
		/*last cu command was a CU_RESMUE if this is a 558 or newer we don't need to
		 * wait for command word to clear, we reach here only if we are bachlor
		 */
		e100_exec_cmd( SCB_CUC_RESUME);
		private_data.next_cu_cmd = RESUME_WAIT;
		break;

	case RESUME_WAIT:
		if ((private_data.flags & IS_ICH) &&
		    (private_data.cur_line_speed == 10) &&
		    (private_data.cur_dplx_mode == HALF_DUPLEX)) {
			e100_wait_exec_simple( SCB_CUC_NOOP);
			udelay(1);
		}
		if ((e100_wait_exec_simple( SCB_CUC_RESUME)) &&
		    (private_data.flags & IS_BACHELOR) && (!(private_data.flags & IS_ICH))) {
			private_data.next_cu_cmd = RESUME_NO_WAIT;
		}
		break;

	case START_WAIT:
		// The last command was a non_tx CU command
		if (!e100_wait_cus_idle())
			rtl_printf(KERN_DEBUG
			       "e100: cu_start: timeout waiting for cu\n");


		if (!e100_wait_exec_cmplx( (u32) (tcb->tcb_phys),
					  SCB_CUC_START, CB_TRANSMIT)) {
			rtl_printf(KERN_DEBUG
			       "e100: cu_start: timeout waiting for scb\n");
			e100_exec_cmplx( (u32) (tcb->tcb_phys),
					SCB_CUC_START);
			ret = false;
		}

		//rtl_printf(KERN_ERR "***In e100_start_cu(), START_WAIT mode \n");
		private_data.next_cu_cmd = RESUME_WAIT;

		break;
	}

	/* save the last tcb */
	private_data.last_tcb = tcb;

	return ret;
}

/* ====================================================================== */
/* hw                                                                     */
/* ====================================================================== */

/**
 * e100_selftest - perform H/W self test
 * @private_data: atapter's private data struct
 * @st_timeout: address to return timeout value, if fails
 * @st_result: address to return selftest result, if fails
 *
 * This routine will issue PORT Self-test command to test the e100.
 * The self-test will fail if the adapter's master-enable bit is not
 * set in the PCI Command Register, or if the adapter is not seated
 * in a PCI master-enabled slot. we also disable interrupts when the
 * command is completed.
 *
 * Returns:
 *      true: if adapter passes self_test
 *      false: otherwise
 */
unsigned char
e100_selftest(u32 *st_timeout, u32 *st_result)
{
	u32 selftest_cmd;

	/* initialize the nic state before running test */
	e100_sw_reset( PORT_SOFTWARE_RESET);

	/* Setup the address of the self_test area */
	selftest_cmd = private_data.selftest_phys;

	/* Setup SELF TEST Command Code in D3 - D0 */
	selftest_cmd |= PORT_SELFTEST;

	/* Initialize the self-test signature and results DWORDS */
	private_data.selftest->st_sign = 0;
	private_data.selftest->st_result = 0xffffffff;

	/* Do the port command */
	writel(selftest_cmd, &private_data.scb->scb_port);
	readw(&(private_data.scb->scb_status));	/* flushes last write, read-safe */

	/* Wait at least 10 milliseconds for the self-test to complete */
	mdelay(11);
	/* disable interrupts since they are enabled */
	/* after device reset during selftest        */
	e100_disable_clear_intr();

	/* if The First Self Test DWORD Still Zero, We've timed out. If the
	 * second DWORD is not zero then we have an error. */
	if ((private_data.selftest->st_sign == 0) || (private_data.selftest->st_result != 0)) {

		if (st_timeout)
			*st_timeout = !(le32_to_cpu(private_data.selftest->st_sign));

		if (st_result)
			*st_result = le32_to_cpu(private_data.selftest->st_result);

		return false;
	}

	return true;
}

/**
 * e100_setup_iaaddr - issue IA setup sommand
 * @private_data: atapter's private data struct
 * @eaddr: new ethernet address
 *
 * This routine will issue the IA setup command. This command
 * will notify the 82557 (e100) of what its individual (node)
 * address is. This command will be executed in polled mode.
 *
 * Returns:
 *      true: if the IA setup command was successfully issued and completed
 *      false: otherwise
 */
unsigned char
e100_setup_iaaddr(u8 *eaddr)
{
	unsigned int i;
	cb_header_t *ntcb_hdr;
	unsigned char res;
	nxmit_cb_entry_t *cmd;

	if ((cmd = e100_alloc_non_tx_cmd()) == NULL) {
		res = false;
		goto exit;
	}

	ntcb_hdr = (cb_header_t *) cmd->non_tx_cmd;
	ntcb_hdr->cb_cmd = 0;
	ntcb_hdr->cb_cmd = __constant_cpu_to_le16(CB_IA_ADDRESS);

	for (i = 0; i < ETH_ALEN; i++) {
		(cmd->non_tx_cmd)->ntcb.setup.ia_addr[i] = eaddr[i];
	}

	res = e100_exec_non_cu_cmd( cmd);
	if (!res)
		rtl_printf(KERN_WARNING "e100: %s: IA setup failed\n", 
		       private_data.ifname);

exit:
	return res;
}

/**
 * e100_start_ru - start the RU if needed
 * @private_data: atapter's private data struct
 *
 * This routine checks the status of the 82557's receive unit(RU),
 * and starts the RU if it was not already active.  However,
 * before restarting the RU, the driver gives the RU the buffers
 * it freed up during the servicing of the ISR. If there are
 * have reached a
 * no resource condition) the RU will not be started till the
 * next ISR.
 */
void
e100_start_ru()
{
	struct rx_list_elem *rx_struct = private_data.rvhead;

	spin_lock(&private_data.bd_lock);

	if (!e100_wait_exec_cmplx( rx_struct->dma_addr, SCB_RUC_START, 0)) {
		rtl_printf(KERN_DEBUG
		       "e100: start_ru: wait_scb failed\n");
		e100_exec_cmplx( rx_struct->dma_addr, SCB_RUC_START);
	}
	if (private_data.next_cu_cmd == RESUME_NO_WAIT) {
		private_data.next_cu_cmd = RESUME_WAIT;
	}
	spin_unlock(&private_data.bd_lock);
}

/**
 * e100_cmd_complete_location
 * @private_data: atapter's private data struct
 *
 * This routine returns a pointer to the location of the command-complete
 * DWord in the dump statistical counters area, according to the statistical
 * counters mode (557 - basic, 558 - extended, or 559 - TCO mode).
 * See e100_config_init() for the setting of the statistical counters mode.
 */
static u32 *
e100_cmd_complete_location(void)
{
	u32 *cmd_complete;
	max_counters_t *stats = private_data.stats_counters;

	switch (private_data.stat_mode) {
	case E100_EXTENDED_STATS:
		cmd_complete =
			(u32 *) &(((err_cntr_558_t *) (stats))->cmd_complete);
		break;

	case E100_TCO_STATS:
		cmd_complete =
			(u32 *) &(((err_cntr_559_t *) (stats))->cmd_complete);
		break;

	case E100_BASIC_STATS:
	default:		
		cmd_complete =
			(u32 *) &(((err_cntr_557_t *) (stats))->cmd_complete);
		break;
	}

	return cmd_complete;
}

/**
 * e100_clr_cntrs - clear statistics counters
 * @private_data: atapter's private data struct
 *
 * This routine will clear the adapter error statistic counters.
 *
 * Returns:
 *      true: if successfully cleared stat counters
 *      false: otherwise
 */
static unsigned char
e100_clr_cntrs()
{
	volatile u32 *pcmd_complete;

	/* clear the dump counter complete word */
	pcmd_complete = e100_cmd_complete_location();
	*pcmd_complete = 0;
	wmb();

	if (!e100_wait_exec_cmplx( private_data.stat_cnt_phys, SCB_CUC_DUMP_ADDR, 0))
		return false;

	/* wait 10 microseconds for the command to complete */
	udelay(10);

	if (!e100_wait_exec_simple( SCB_CUC_DUMP_RST_STAT))
		return false;

	if (private_data.next_cu_cmd == RESUME_NO_WAIT) {
		private_data.next_cu_cmd = RESUME_WAIT;
	}

	return true;
}

/**
 * e100_exec_non_cu_cmd
 * @private_data: atapter's private data struct
 * @command: the non-cu command to execute
 *
 * This routine will submit a command block to be executed,
 *delay exe or exe immediately 
*/
unsigned char
e100_exec_non_cu_cmd( nxmit_cb_entry_t *command)
{
	cb_header_t *ntcb_hdr;
	unsigned long lock_flag;
	//unsigned long expiration_time;
	unsigned char rc = true;
	u8 sub_cmd;

	ntcb_hdr = (cb_header_t *) command->non_tx_cmd;	/* get hdr of non tcb cmd */
	sub_cmd = cpu_to_le16(ntcb_hdr->cb_cmd);

	/* Set the Command Block to be the last command block */
	/*set EL is VIP since CU will go from active to idle*/
	/*after the completion*/
	ntcb_hdr->cb_cmd |= __constant_cpu_to_le16(CB_EL_BIT);
	ntcb_hdr->cb_status = 0;
	ntcb_hdr->cb_lnk_ptr = 0;

	wmb();

	spin_lock_bh(&(private_data.bd_non_tx_lock));

	/*if has tx command, delay exe*/
	if (private_data.last_tcb) {
		rmb();
		while ((private_data.last_tcb->tcb_hdr.cb_status &
		     __constant_cpu_to_le16(CB_STATUS_COMPLETE)) == 0)
		  {
		    rmb();
		    mdelay(10);		
		  }	
		//goto delayed_exec;
	}
	/*if CU is busy, delay exe*/
	while ((readw(&private_data.scb->scb_status) & SCB_CUS_MASK) == SCB_CUS_ACTIVE) {
	  rmb();	  
	  mdelay(10);
	  //goto delayed_exec;
	}

	spin_lock_irqsave(&private_data.bd_lock, lock_flag);
	/* otherwise, exe non-tx command*/
	if (!e100_wait_exec_cmplx( command->dma_addr, SCB_CUC_START, sub_cmd)) {
		spin_unlock_irqrestore(&(private_data.bd_lock), lock_flag);
		rc = false;
		goto exit;
	}

	private_data.next_cu_cmd = START_WAIT;
	spin_unlock_irqrestore(&(private_data.bd_lock), lock_flag);

	/* now wait for completion of non-cu CB up to 20 msec */
	rmb();
	while (!(ntcb_hdr->cb_status &
		     __constant_cpu_to_le16(CB_STATUS_COMPLETE))) {
	  rmb();
	  mdelay(20);
	}

exit:
	e100_free_non_tx_cmd( command);

	spin_unlock_bh(&(private_data.bd_non_tx_lock));
	return rc;

}

/**
 * e100_sw_reset
 * @private_data: atapter's private data struct
 * @reset_cmd: s/w reset or selective reset
 *
 * This routine will issue a software reset to the adapter. It 
 * will also disable interrupts, as the are enabled after reset.
 */
void
e100_sw_reset( u32 reset_cmd)
{
	/* Do  a selective reset first to avoid a potential PCI hang */
	writel(PORT_SELECTIVE_RESET, &private_data.scb->scb_port);
	readw(&(private_data.scb->scb_status));	/* flushes last write, read-safe */

	/* wait for the reset to take effect */
	udelay(20);
	if (reset_cmd == PORT_SOFTWARE_RESET) {
		writel(PORT_SOFTWARE_RESET, &private_data.scb->scb_port);

		/* wait 20 micro seconds for the reset to take effect */
		udelay(20);
	}

	/* Mask off our interrupt line -- it is unmasked after reset */
	e100_disable_clear_intr();
#ifdef E100_CU_DEBUG	
	private_data.last_cmd = 0;
	private_data.last_sub_cmd = 0;
#endif	
}

/**
 * e100_load_microcode - Download microsocde to controller.
 * @private_data: atapter's private data struct
 *
 * This routine downloads microcode on to the controller. This
 * microcode is available for the 82558/9, 82550. Currently the
 * microcode handles interrupt bundling and TCO workaround.
 *
 * Returns:
 *      true: if successfull
 *      false: otherwise
 */
static unsigned char
e100_load_microcode()
{
	static struct {
		u8 rev_id;
		u32 ucode[UCODE_MAX_DWORDS + 1];
		int timer_dword;
		int bundle_dword;
		int min_size_dword;
	} ucode_opts[] = {
		{ D101A4_REV_ID,
		  D101_A_RCVBUNDLE_UCODE,
		  D101_CPUSAVER_TIMER_DWORD,
		  D101_CPUSAVER_BUNDLE_DWORD,
		  D101_CPUSAVER_MIN_SIZE_DWORD },
		{ D101B0_REV_ID,
		  D101_B0_RCVBUNDLE_UCODE,
		  D101_CPUSAVER_TIMER_DWORD,
		  D101_CPUSAVER_BUNDLE_DWORD,
		  D101_CPUSAVER_MIN_SIZE_DWORD },
		{ D101MA_REV_ID,
		  D101M_B_RCVBUNDLE_UCODE,
		  D101M_CPUSAVER_TIMER_DWORD,
		  D101M_CPUSAVER_BUNDLE_DWORD,
		  D101M_CPUSAVER_MIN_SIZE_DWORD },
		{ D101S_REV_ID,
		  D101S_RCVBUNDLE_UCODE,
		  D101S_CPUSAVER_TIMER_DWORD,
		  D101S_CPUSAVER_BUNDLE_DWORD,
		  D101S_CPUSAVER_MIN_SIZE_DWORD },
		{ D102_REV_ID,
		  D102_B_RCVBUNDLE_UCODE,
		  D102_B_CPUSAVER_TIMER_DWORD,
		  D102_B_CPUSAVER_BUNDLE_DWORD,
		  D102_B_CPUSAVER_MIN_SIZE_DWORD },
		{ D102C_REV_ID,
		  D102_C_RCVBUNDLE_UCODE,
		  D102_C_CPUSAVER_TIMER_DWORD,
		  D102_C_CPUSAVER_BUNDLE_DWORD,
		  D102_C_CPUSAVER_MIN_SIZE_DWORD },
		{ D102E_REV_ID,
		  D102_E_RCVBUNDLE_UCODE,
		  D102_E_CPUSAVER_TIMER_DWORD,
		  D102_E_CPUSAVER_BUNDLE_DWORD,
		  D102_E_CPUSAVER_MIN_SIZE_DWORD },
		{ 0, {0}, 0, 0, 0}
	}, *opts;

	opts = ucode_opts;
	/* User turned ucode loading off */
	if (!(private_data.params.b_params & PRM_UCODE))
		return false;

	/* These controllers do not need ucode */
	if (private_data.flags & IS_ICH)
		return false;


	/* Search for ucode match against h/w rev_id */
	while (opts->rev_id) {
		if (private_data.rev_id == opts->rev_id) {
			int i;
			u32 *ucode_dword;
			load_ucode_cb_t *ucode_cmd_ptr;
			nxmit_cb_entry_t *cmd = e100_alloc_non_tx_cmd();

			if (cmd != NULL) {
				ucode_cmd_ptr =
					(load_ucode_cb_t *) cmd->non_tx_cmd;
				ucode_dword = ucode_cmd_ptr->ucode_dword;
				rtl_printf(KERN_ERR "***e100_alloc_non_tx_cmd succeed\n");
			} else {
			  rtl_printf(KERN_ERR "***e100_alloc_non_tx_cmd failed\n");
				return false;
			}

			memcpy(ucode_dword, opts->ucode, sizeof (opts->ucode));

			/* Insert user-tunable settings */
			ucode_dword[opts->timer_dword] &= 0xFFFF0000;
			ucode_dword[opts->timer_dword] |=
				(u16) private_data.params.IntDelay;
			ucode_dword[opts->bundle_dword] &= 0xFFFF0000;
			ucode_dword[opts->bundle_dword] |=
				(u16) private_data.params.BundleMax;
			ucode_dword[opts->min_size_dword] &= 0xFFFF0000;
			ucode_dword[opts->min_size_dword] |=
				(private_data.params.b_params & PRM_BUNDLE_SMALL) ?
				0xFFFF : 0xFF80;

			for (i = 0; i < UCODE_MAX_DWORDS; i++)
				cpu_to_le32s(&(ucode_dword[i]));

			ucode_cmd_ptr->load_ucode_cbhdr.cb_cmd =
				__constant_cpu_to_le16(CB_LOAD_MICROCODE);

			return e100_exec_non_cu_cmd( cmd);
		}
		opts++;
	}

	return false;
}

/***************************************************************************/
/***************************************************************************/
/*       EEPROM  Functions                                                 */
/***************************************************************************/

/* Read PWA (printed wired assembly) number */
void e100_rd_pwa_no()
{
	private_data.pwa_no = e100_eeprom_read( EEPROM_PWA_NO);
	private_data.pwa_no <<= 16;
	private_data.pwa_no |= e100_eeprom_read( EEPROM_PWA_NO + 1);
}

/* Read the permanent ethernet address from the eprom. */
void 
e100_rd_eaddr()
{
	int i;
	u16 eeprom_word;

	for (i = 0; i < 6; i += 2) {
		eeprom_word =
			e100_eeprom_read(
					 EEPROM_NODE_ADDRESS_BYTE_0 + (i / 2));

		private_data.dev_addr[i] =
			private_data.perm_node_address[i] = (u8) eeprom_word;
		private_data.dev_addr[i + 1] =
			private_data.perm_node_address[i + 1] = (u8) (eeprom_word >> 8);
	}
}

/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/*       Auxilary Functions                                                */
/***************************************************************************/

/* Print the board's configuration */
void rt_e100_print_brd_conf()
{
  /*Print the string if checksum Offloading was enabled*/ 
  if (private_data.flags & DF_CSUM_OFFLOAD)
    rtl_printf(KERN_NOTICE "  Hardware receive checksums enabled\n");
  else {
    if (private_data.rev_id >= D101MA_REV_ID) 
      rtl_printf(KERN_NOTICE "  Hardware receive checksums disabled\n");
  }
  
  if ((private_data.flags & DF_UCODE_LOADED))
    rtl_printf(KERN_NOTICE "  cpu cycle saver enabled\n");

}

/**
 * e100_pci_setup - setup the adapter's PCI information
 * @pcid: adapter's pci_dev struct
 * @private_data: atapter's private data struct
 *
 * This routine sets up all PCI information for the adapter. It enables the bus
 * master bit (some BIOS don't do this), requests memory ans I/O regions, and
 * calls ioremap() on the adapter's memory region.
 *
 * Returns:
 *      true: if successfull
 *      false: otherwise
 */
int 
e100_pci_setup(struct pci_dev *pcid)
{

	int rc = 0;

	if ((rc = pci_enable_device(pcid)) != 0) {
	  rtl_printf("******pci_enable_device is not successful\n");
	  goto err;
	}

	/* dev and ven ID have already been checked so it is our device */
	pci_read_config_byte(pcid, PCI_REVISION_ID, (u8 *) &(private_data.rev_id));

	/* address #0 is a memory region */
	private_data.mem_start = pci_resource_start(pcid, 0);
	private_data.mem_end = private_data.mem_start + sizeof (scb_t);

	/* address #1 is a IO region */
	private_data.base_addr = pci_resource_start(pcid, 1);

	if ((rc = pci_request_regions(pcid, ETH_NAME)) != 0) {
	  rtl_printf("******pci_request_regions is not successful\n");
	  goto err_disable;
	}

	pci_enable_wake(pcid, 0, 0);

	/* if Bus Mastering is off, turn it on! */
	pci_set_master(pcid);

	/* address #0 is a memory mapping */
	private_data.scb = (scb_t *) ioremap_nocache(private_data.mem_start, sizeof (scb_t));

	if (!private_data.scb) {
		rtl_printf(KERN_ERR "e100: %s: Failed to map PCI address 0x%lX\n",
		       private_data.ifname, pci_resource_start(pcid, 0));
		rc = -ENOMEM;
		goto err_region;
	}

	return 0;

err_region:
	pci_release_regions(pcid);
err_disable:
	pci_disable_device(pcid);
err:
	return rc;
}

void
e100_isolate_driver()
{
  e100_disable_clear_intr();
  private_data.last_tcb = NULL;
  e100_sw_reset( PORT_SELECTIVE_RESET);
}


static inline int 
e100_10BaseT_adapter(void)
{
	return ((private_data.pdev->device == 0x1229) &&
		(private_data.pdev->subsystem_vendor == 0x8086) &&
		(private_data.pdev->subsystem_device == 0x0003));
}

static void
e100_get_speed_duplex_caps()
{
	u16 status;

	e100_mdi_read( MII_BMSR, private_data.phy_addr, &status);

	private_data.speed_duplex_caps = 0;

	private_data.speed_duplex_caps |=
		(status & BMSR_ANEGCAPABLE) ? SUPPORTED_Autoneg : 0;

	private_data.speed_duplex_caps |=
		(status & BMSR_10HALF) ? SUPPORTED_10baseT_Half : 0;

	private_data.speed_duplex_caps |=
		(status & BMSR_10FULL) ? SUPPORTED_10baseT_Full : 0;

	private_data.speed_duplex_caps |=
		(status & BMSR_100HALF) ? SUPPORTED_100baseT_Half : 0;

	private_data.speed_duplex_caps |=
		(status & BMSR_100FULL) ? SUPPORTED_100baseT_Full : 0;

	if (IS_NC3133())
		private_data.speed_duplex_caps =
			(SUPPORTED_FIBRE | SUPPORTED_100baseT_Full);
	else
		private_data.speed_duplex_caps |= SUPPORTED_TP;

	if ((status == 0xFFFF) && e100_10BaseT_adapter()) {
		private_data.speed_duplex_caps =
			(SUPPORTED_10baseT_Half | SUPPORTED_TP);
	} else {
		private_data.speed_duplex_caps |= SUPPORTED_MII;
	}

}

nxmit_cb_entry_t *
e100_alloc_non_tx_cmd()
{
	nxmit_cb_entry_t *non_tx_cmd_elem;

	if (!(non_tx_cmd_elem = (nxmit_cb_entry_t *)
	      kmalloc(sizeof (nxmit_cb_entry_t), GFP_ATOMIC))) {
		return NULL;
	}
	non_tx_cmd_elem->non_tx_cmd =
		pci_alloc_consistent(private_data.pdev, sizeof (nxmit_cb_t),
				     &(non_tx_cmd_elem->dma_addr));
	if (non_tx_cmd_elem->non_tx_cmd == NULL) {
		kfree(non_tx_cmd_elem);
		return NULL;
	}

	return non_tx_cmd_elem;
}

void
e100_free_non_tx_cmd(
		     nxmit_cb_entry_t *non_tx_cmd_elem)
{
	pci_free_consistent(private_data.pdev, sizeof (nxmit_cb_t),
			    non_tx_cmd_elem->non_tx_cmd,
			    non_tx_cmd_elem->dma_addr);
	kfree(non_tx_cmd_elem);
}



