/*
 * Copywrite 2002 Der Herr Hofrat
 * License GPL V2
 * Author der.herr@hofr.at
 */
/*
 * simple fifo example - just write the current rt_time
 * to a fifo in a loop.
 * Note that the fifo is created by passing O_CREAT to open - this MUST
 * be called in init_module if O_CREAT is passed. Also note that the
 * return value of the thread is collected in pthread join and that 
 * the code after the while(!stop) is executed on exit. The while loop
 * of the main thread exits in a synchronous manner and can't be 
 * terminated in the middle of the loops as with asynchrnous 
 * pthread_cancel() or non-posix pthread_delete_np().
 */

#include <rtl.h>
#include <rtl_fifo.h>
#include <time.h>
#include <rtl_sched.h>
#include <rtl_sync.h>
#include <pthread.h>
#include <unistd.h>
#include <errno.h>

MODULE_AUTHOR("Der Herr Hofrat");
MODULE_LICENSE("GPL v2");
MODULE_DESCRIPTION("simple (posix) fifo");

#define FIFO_SIZE 4000
#define FIFO_NR 0
#define DELAY 500000000 /* .5 seconds  */

pthread_t thread;
int fd;
int stop = 0; 

void *thread_code(void *param)
{
	hrtime_t now;
	struct timespec ts;
	int n = 0;
	unsigned long loops = 0;
	clock_gettime(CLOCK_REALTIME,&ts);

        while (!stop) {
		timespec_add_ns(&ts,DELAY);
		clock_nanosleep(CLOCK_REALTIME, TIMER_ABSTIME, &ts, NULL);
		now = clock_gethrtime(CLOCK_MONOTONIC);
		n = write (fd, &now, sizeof(now));
		if(n != sizeof(now)){
			rtl_printf("write to fifo failed\n");
			return (void *)-1 ;
		}
		loops++;
	};

	return (void *)loops;
}

int init_module(void)
{
	pthread_attr_t attr;
	struct sched_param sched_param;
        int ret;

	fd = open ("/dev/rtf0",O_NONBLOCK|O_CREAT);
	if (fd < 0) {
		printk ("failed to open /dev/rtf%d\n",FIFO_NR);
		return -1;
	}

	pthread_attr_init (&attr);
	sched_param.sched_priority = 1;
	pthread_attr_setschedparam (&attr, &sched_param);
	ret = pthread_create (&thread, &attr, thread_code, (void *)0);
	if (ret) {
		printk ("failed to create thread (%d)\n", ret);
		return -1;
	} else {
		printk("thread created sucessfully\n");
	}

	return 0;
}

void cleanup_module(void)
{
	unsigned long loops = 0;
	stop = 1;
	pthread_join (thread,(void *)&loops);
	close (fd);
        printk ("Removed simple (posix) fifo after %ld loops\n",loops);
}
