/* ugly - pure posix - variant of hello.c - it makes clear why
 * adding pthread_make_periodic_np pthread_wait_np makes a lot fo sens..
 *
 * posix provides no direct mechanism for periodic threads
 * so we use a periodic posix intervall-timer and let the handler 
 * of this timer signal a wakeup to the thread - note the somwhat
 * complex way for suspending the thread - simpler ways are
 * posible (see sem_hello.c) but this should make the sync
 * requirements clear (I hope).
 * 
 * hofrat.
 */

#ifdef CONFIG_OC_PTIMERS

#include <rtl.h>
#include <pthread.h>
#include <time.h>
#include <signal.h>

MODULE_LICENSE("GPL v2");
MODULE_AUTHOR("der.herr@hofr.at");
MODULE_DESCRIPTION("pure posix periodic thread - using signals and timers");

static pthread_t thread;
static timer_t timer;

#define PERIODIC_CLOCK_MODE

void
timer_intr(int sig)
{
  pthread_kill(pthread_self(),RTL_SIGNAL_WAKEUP);
  pthread_yield();
}

void
*start_routine(void *arg)
{
  struct sched_param p;
  struct itimerspec new_setting,old_setting;
  struct sigaction sa;
  hrtime_t now=0,last=0;
  int err=0;

  p . sched_priority = 1;
  pthread_setschedparam (pthread_self(), SCHED_FIFO, &p);
 
  /* set up handler for the posix timer interrupt */
  sa.sa_handler=timer_intr;
  sa.sa_mask=0;
  sa.sa_flags=0;

  /* set up the intervall timer */
  new_setting.it_interval.tv_sec=0; /* periodic */
  new_setting.it_interval.tv_nsec=1000000LL; /* period in ns -> 1kHz */

  new_setting.it_value.tv_sec=1;
  new_setting.it_value.tv_nsec=0;


  /* bind timer handler to signal */
  if ((err=sigaction(RTL_SIGUSR1,&sa,NULL))<0 ){
    rtl_printf("sigaction failed for RTL_SIGUSR1 (%d)\n",err);
  }

  err=timer_settime(timer,0,&new_setting,&old_setting);
  
  while (1) {
    last=now; 
    now=clock_gethrtime(CLOCK_REALTIME);
   
    rtl_printf("elapsed_time = %Ld\n",(long long)now-last);

    pthread_kill(pthread_self(),RTL_SIGNAL_SUSPEND); 
    pthread_yield();
    pthread_testcancel();  /* honor any pending cancelation here */
  }
  return 0; 
}


int 
init_module(void) {
	
  sigevent_t signal; 

  /* timer should signal expiraton via RTL_SIGUSR1  */
  signal.sigev_notify=SIGEV_SIGNAL;
  signal.sigev_signo=RTL_SIGUSR1;

  if(!(timer_create(CLOCK_REALTIME,&signal,&timer))){
     printk("Timer created\n");
  } else {
     printk("Error creating timer \n");
     return -1;
  }
  
  return pthread_create (&thread, NULL, start_routine,(void *) 0);
}

void 
cleanup_module(void) {
    timer_delete(timer);
    printk("Timer deleted\n");
    pthread_cancel(thread);
    pthread_join(thread,NULL);
    printk("thread terminated\n");
}

#endif /* #ifdef CONFIG_OC_PTIMERS */
