/* pure posix solution using cond_nanosleep
 * Simply add the period to the timeval and sleep until the next
 * period elapsed - clock_nanosleep calls the scheduler and is 
 * a cancelation point - so this is a safe way of implementing
 * periodic threads.
 * hofrat
 */

#include <rtl.h>
#include <pthread.h>
#include <time.h>
#include <signal.h>

MODULE_LICENSE("GPL v2");
MODULE_AUTHOR("der.herr@hofr.at");
MODULE_DESCRIPTION("pure posix periodic thread - using clock_nanosleep");

static pthread_t thread;
int stop=0;

#define DELAY 1000000 /* 1 millisecond */

void
*start_routine(void *arg)
{
  struct sched_param p;
  struct timespec t;
  hrtime_t now=0,last=0;

  p . sched_priority = 1;
  pthread_setschedparam (pthread_self(), SCHED_FIFO, &p);
  clock_gettime(CLOCK_REALTIME,&t);
 
  
  while (!stop) {
    last=now; 
    now=clock_gethrtime(CLOCK_REALTIME);
    rtl_printf("elapsed_time = %Ld\n",(long long)now-last);
    timespec_add_ns(&t,DELAY);
    clock_nanosleep(CLOCK_REALTIME, TIMER_ABSTIME, &t, NULL);
  }
  return 0; 
}


int 
init_module(void) 
{
  return pthread_create (&thread, NULL, start_routine,(void *) 0);
}

void 
cleanup_module(void) 
{
    stop=1;	/* ugly sync with global variable */
    pthread_cancel(thread);
    pthread_join(thread,NULL);
    printk("thread terminated\n");
}
