/* third variant of a posix complient periodic thread
 * what this does is initialize a conditional variable that is not actually
 * used here (but could be to allow another thread to signal this thread to
 * return early !), and then we simply use the pthread_cond_timedwait() to
 * wait for the threads period.
 * 
 * thanks to Herman Bruyninckx for posting this solution to the RTAI mailing
 * list - stolen by hofrat ;)
 */

#include <rtl.h>
#include <pthread.h>
#include <time.h>
#include <signal.h>

MODULE_LICENSE("GPL v2");
MODULE_AUTHOR("der.herr@hofr.at");
MODULE_DESCRIPTION("pure posix periodic thread - using condvar");

static pthread_t thread;
int stop=0;
static pthread_mutex_t mutex /* = PTHREAD_MUTEX_INITIALIZER */;
static pthread_cond_t cond;

#define TH_PERIOD 1000000 /* 1 millisecond */

void
*start_routine(void *arg)
{
  struct sched_param p;
  struct timespec t;
  hrtime_t now=0,last=0;
  int ret;

  p . sched_priority = 1;
  pthread_setschedparam (pthread_self(), SCHED_FIFO, &p);
  clock_gettime(CLOCK_REALTIME,&t);
 
  
  while (!stop) {
    pthread_mutex_lock (&mutex);
    ret = pthread_cond_timedwait (&cond, &mutex, &t);
    pthread_mutex_unlock (&mutex);
    timespec_add_ns(&t,TH_PERIOD);

    /* do the actual work */
    last=now; 
    now=clock_gethrtime(CLOCK_REALTIME);
    rtl_printf("elapsed_time = %Ld\n",(long long)now-last);
  }
  return 0; 
}


int 
init_module(void) 
{
  return pthread_create (&thread, NULL, start_routine,(void *) 0);
}

void 
cleanup_module(void) 
{
    stop=1;	/* ugly sync with global variable */
    pthread_cancel(thread);
    pthread_join(thread,NULL);
    printk("thread terminated\n");
}
