/* vim: set ts=4: */
/*
 * Copywrite 2002 Der Herr Hofrat
 * License GPL V2
 * Author der.herr@hofr.at
 */
/*
 * example of a simple PWM
 */
#include <rtl.h>
#include <rtl_fifo.h>
#include <time.h>
#include <rtl_sched.h>
#include <rtl_sync.h>
#include <pthread.h>
#include <posix/unistd.h>
#include <asm/io.h> /* outb */

MODULE_LICENSE("GPL v2");
MODULE_AUTHOR("Der Herr Hofrat <der.herr@hofr.at>");
MODULE_DESCRIPTION("Trivial PWM for the parport");

#define LPT 0x378
#define LPT_CNTRL LPT+2 

pthread_t pwm_thread;
void *pwm_code(void *);
int fd;
int stop = 0;
#define BASE_NS 10000 /* 1% set to 10 micro seconds */
#define DELAY_NS 100*BASE_NS /* 100% on */
unsigned long duty_cycle=50*BASE_NS; /* 50% duty cycle */


void *pwm_code(void *arg){
	struct timespec t;
	unsigned char pins=0xff;
	clock_gettime(CLOCK_REALTIME,&t);
	while(!stop){
		outb(pins,LPT);
		pins = ~pins;
		timespec_add_ns(&t,DELAY_NS-duty_cycle);
		clock_nanosleep(CLOCK_REALTIME, TIMER_ABSTIME, &t, NULL);
		outb(pins,LPT);
		pins = ~pins;
		timespec_add_ns(&t,duty_cycle);
		clock_nanosleep(CLOCK_REALTIME, TIMER_ABSTIME, &t, NULL);
	}
	return (void *)stop;
}

int param_handler(unsigned int fifo)
{
	int err;

	while((err = rtf_get(0, &duty_cycle, sizeof(duty_cycle))) == sizeof(duty_cycle)){
		if(duty_cycle < BASE_NS){
			duty_cycle=BASE_NS;
		} else {
			duty_cycle*=BASE_NS ;
		}
		rtl_printf("duty cycle set to %d\n",(int)duty_cycle);
	}
	if (err != 0) {
		return -EINVAL;
	}
	return 0;
}

int init_module(void){

	rtf_create(0,4000);	
	if ( (fd = open("/dev/rtf0",O_RDONLY | O_NONBLOCK )) < 0)
	{
		printk("Open fifo failed %d\n",errno);
		return -1;
	}
	if(pthread_create(&pwm_thread,NULL,pwm_code,NULL))
	{
		close(fd);
		printk("Pthread_create failed\n");
		return -1; 
	}
	rtf_create_handler(0, &param_handler); 
	return 0;
}

void cleanup_module(void){ 
	stop = 1;
	pthread_join(pwm_thread,NULL);
	close(fd);
}

