/*
 * POSIX.1 Signals test program
 *
 * Written by J. Vidal
 * Copyright (C) Dec, 2002 OCERA Consortium.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2.
 *
 * Testing pthread_kill real time constraints.
 *
 */
#include <rtl.h>
#include <pthread.h>

#define NTASKS 2
static pthread_t thread[NTASKS];

static void signal_handler(int signal){
  int i;

  rtl_printf("Signal handler called for signal %d\n",signal);
  rtl_printf("TEST SUCCED!!!!! A lower priority thread has send me a signal\n");
  rtl_printf("and I have taken over the CPU\n");
  rtl_printf("Now suspending all threads\n");
  
  for (i=0;i<NTASKS;i++) pthread_suspend_np(thread[i]);
}

static void *hight_prio_routine(void *arg){
  struct sigaction sa;
  int err;
  struct sched_param p;
  rtl_sigset_t mask;

  p.sched_priority=100;
  pthread_setschedparam (pthread_self(), SCHED_FIFO, &p);
  
  sa.sa_handler=signal_handler;
  sa.sa_mask=0;
  sa.sa_flags=0;
  
  if ((err=sigaction(RTL_SIGUSR1,&sa,NULL))<0 ){
    rtl_printf("sigaction(RTL_SIGUSR1,&sa,NULL) FAILING, err:%d.\n",err);
    pthread_exit(NULL);
  }
  
  rtl_sigfillset(&mask);
  rtl_sigdelset(&mask,RTL_SIGUSR1);
  sigsuspend(&mask);

 return 0;
}


static void *start_routine(void *arg)
{
  struct sched_param p;
  int param;

  param=(unsigned) arg;
  p . sched_priority = param;
  pthread_setschedparam (pthread_self(), SCHED_FIFO, &p);
  
  rtl_printf("Before sending a signal to a higher priority thread\n");
  pthread_kill(thread[NTASKS-1],RTL_SIGUSR1);
  rtl_printf("TEST FAILED !!!!!!!! Not loosing the CPU\n");
  
  pthread_exit(NULL);
  return 0;
}

int init_module(void) {
  int i;
 
  // This thread will be signaled by a lower priority thread.
  pthread_create(&thread[NTASKS-1],NULL,hight_prio_routine,(void *)NULL);
  
  for (i=0;i<NTASKS-1;i++)
    pthread_create (&thread[i], NULL, start_routine, (int *) i);
  
  return 0;
}

void cleanup_module(void) {
  int i;
  for (i=0;i<NTASKS;i++)
    pthread_delete_np (thread[i]);
}
