/*
 * RTLinux FIFOs
 *
 *  Copyright (C) 1999-2007  Wind River Systems, Inc. (http://www.windriver.com/)
 *  Written by Michael Barabanov <Michael.Barabanov@windriver.com>, 1995-2000
 */

#ifndef __RTF__
#define __RTF__

#include <rtl_conf.h>

#define RTF_NO (CONFIG_RTL_NFIFOS)
#define RTL_MAX_USER_FIFO RTF_NO

#define RTF_MAX_FIFO (RTL_MAX_USER_FIFO + 8)
#define RTF_SETSIZE 1

#ifdef __KERNEL__

#include <linux/slab.h>
/* these macros are just shifted from fifos/rtl_fifo.c
 * to make then available to users for convenience
 */
struct rt_fifo_struct {
	int allocated;
	int bidirectional;
	int user_open;
	struct task_struct *opener;
	char *base;
	int bufsize;
	int start;
	int len;
	spinlock_t fifo_spinlock;
	int (*user_handler) (unsigned int fifo);
	int (*rt_handler) (unsigned int fifo);
	int (*user_ioctl)(unsigned int fifo, unsigned int cmd, unsigned long arg);
	struct module *creator;
#if LINUX_VERSION_CODE >= 0x020300
	wait_queue_head_t wait;
#else
	struct wait_queue *wait;
#endif
};

extern struct rt_fifo_struct rtl_fifos[RTF_MAX_FIFO];

#define RTF_ADDR(minor)         (&rtl_fifos[minor])
#define RTF_BASE(minor)         (RTF_ADDR(minor)->base)
#define RTF_BUF(minor)          (RTF_ADDR(minor)->bufsize)
#define RTF_START(minor)        (RTF_ADDR(minor)->start)
#define RTF_LEN(minor)          (RTF_ADDR(minor)->len)
#define RTF_FREE(minor)         (RTF_BUF(minor) - RTF_LEN(minor))
#define RTF_END(minor) \
	({ typeof(minor) _minor = (minor); \
	int _pos = (RTF_START(minor)+RTF_LEN(minor)); \
	(_pos) < RTF_BUF(_minor) ? (_pos) : (_pos) - RTF_BUF(_minor); })
#define RTF_EMPTY(minor)        (RTF_LEN(minor)==0)
#define RTF_FULL(minor)         (RTF_FREE(minor)==0)

extern int rtf_init(void);

extern int rtf_create_handler(unsigned int fifo,
		int (*handler)(unsigned int fifo));


extern int rtf_create_rt_handler(unsigned int fifo,
		int (*handler)(unsigned int fifo));


struct module;
extern struct module __this_module;
extern int __rtf_create(unsigned int fifo, int size, struct module *mod);

#define rtf_create(fifo, size) __rtf_create(fifo, size, &__this_module)

/* bidirectional FIFOs:
 * fifo is for rtf_get (user-space write),
 * fifo_put is for rtf_put (user-space read)
 */ 
extern int rtf_make_user_pair (unsigned int fifo, unsigned int fifo_put);


extern int rtf_link_user_ioctl (unsigned int fifo,
		int (*handler)(unsigned int fifo, unsigned int cmd, unsigned long arg));

extern int rtf_destroy(unsigned int fifo);

extern int rtf_flush(unsigned int fifo);

extern int rtf_isempty(unsigned int fifo);

extern int rtf_isused(unsigned int fifo);

extern int rtf_resize(unsigned int minor, int size);

extern int rtf_put(unsigned int fifo, void * buf, int count);

extern int rtf_get(unsigned int fifo, void * buf, int count);


#define RTFSETSIZE 0
#define RTFMMAPABLE 1

#endif

#endif
