/*
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 * 3. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 */

/*********************************************************************************/
/* This file has been written by Sergio Perez Alcaiz <serpeal@upvnet.upv.es>    */
/*            Departamento de Informtica de Sistemas y Computadores             */
/*            Universidad Politcnica de Valencia                                */
/*            Valencia (Spain)                                                   */
/*                                                                               */
/* The RTL-lwIP project has been supported by the Spanish Government Research    */
/* Office (CICYT) under grant TIC2002-04123-C03-03                               */
/*                                                                               */
/* Copyright (c) March, 2003 SISTEMAS DE TIEMPO REAL EMPOTRADOS, FIABLES Y       */
/* DISTRIBUIDOS BASADOS EN COMPONENTES                                           */
/*                                                                               */
/*  This program is free software; you can redistribute it and/or modify         */
/*  it under the terms of the GNU General Public License as published by         */
/*  the Free Software Foundation; either version 2 of the License, or            */
/*  (at your option) any later version.                                          */
/*                                                                               */
/*  This program is distributed in the hope that it will be useful,              */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                */
/*  GNU General Public License for more details.                                 */
/*                                                                               */
/*  You should have received a copy of the GNU General Public License            */
/*  along with this program; if not, write to the Free Software                  */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA    */
/*                                                                               */
/*  Linking RTL-lwIP statically or dynamically with other modules is making a    */
/*  combined work based on RTL-lwIP.  Thus, the terms and conditions of the GNU  */
/*  General Public License cover the whole combination.                          */
/*                                                                               */
/*  As a special exception, the copyright holders of RTL-lwIP give you           */
/*  permission to link RTL-lwIP with independent modules that communicate with   */
/*  RTL-lwIP solely through the interfaces, regardless of the license terms of   */
/*  these independent modules, and to copy and distribute the resulting combined */
/*  work under terms of your choice, provided that every copy of the combined    */
/*  work is accompanied by a complete copy of the source code of RTL-lwIP (the   */
/*  version of RTL-lwIP used to produce the combined work), being distributed    */
/*  under the terms of the GNU General Public License plus this exception.  An   */
/*  independent module is a module which is not derived from or based on         */
/*  RTL-lwIP.                                                                    */
/*                                                                               */
/*  Note that people who make modified versions of RTL-lwIP are not obligated to */
/*  grant this special exception for their modified versions; it is their choice */
/*  whether to do so.  The GNU General Public License gives permission to        */
/*  release a modified version without this exception; this exception also makes */
/*  it possible to release a modified version which carries forward this         */
/*  exception.                                                                   */
/*********************************************************************************/

// KERNEL INCLUDES
#include <linux/module.h>
#include <linux/kernel.h>
// RTL_LWIP INCLUDES
#include "lwip/sys.h" 
#include "lwip/pbuf.h"
#include "lwip/ip.h"
#include "lwip/udp.h"
#include "lwip/tcp.h"
#include "lwip/tcpip.h"
#include "netif/loopif.h"
#include "netif/rtfifoif.h"
#include "lwip/ip_addr.h"
#include "lwip/mem.h"
#include "lwip/memp.h"
// RTL INCLUDES
#include <rtl_sched.h>
#include <rtl_fifo.h>
#include <rtl_printf.h>


#ifdef RT3COM905C_ISAVAILABLE
#include "netif/rt_3c905x_exports.h"  
#endif
#ifdef RTRTL8139_ISAVAILABLE
#include "netif/rt_rtl8139_exports.h"
#endif

MODULE_AUTHOR("Sergio Perez <serpeal@upvnet.upv.es>");
MODULE_DESCRIPTION("RT-Linux TCP/IP Stack based on lwIP");
MODULE_LICENSE("GPL");

/*-----------------------------------------------------------------------------------*/
static void
tcpip_init_done(void *arg)
{
  sys_sem_t *sem;
  sem = arg;
  sys_sem_signal(*sem);
}

/*-----------------------------------------------------------------------------------*/
static void
main_thread(void *arg){
  struct ip_addr ipaddr, netmask, gw;
  sys_sem_t sem;
  struct netif *ethernetif;

  mem_init();
  memp_init();
  pbuf_init();

  rtl_printf("System initialized.\n");

  netif_init();

  sem = sys_sem_new(0);

  tcpip_init(tcpip_init_done, &sem);

  sys_sem_wait(sem);
  sys_sem_free(sem);

  rtl_printf("TCP/IP initialized.\n");

#ifdef __RTETHERNETOPTS__

  if(RT3COM905C_ISAVAILABLE){

    string2ip_addr(&ipaddr,RT3COM905C_IP, sizeof(RT3COM905C_IP));
    string2ip_addr(&gw,RT3COM905C_GW, sizeof(RT3COM905C_GW));
    string2ip_addr(&netmask,RT3COM905C_NETMASK, sizeof(RT3COM905C_NETMASK));
    
    ethernetif = netif_add(&ipaddr, &netmask, &gw, NULL, rt_3c905cif_init, tcpip_input);
    
    rtl_printf("rt_3com905c interface added to netif list\n");
    
    if(RT3COM905C_ISDEFAULTIF){
      netif_set_default(ethernetif);
      rtl_printf("rt_3com905c set as default\n");
    }
  }

  if(RTRTL8139_ISAVAILABLE){

    string2ip_addr(&ipaddr,RTRTL8139_IP, sizeof(RTRTL8139_IP));
    string2ip_addr(&gw,RTRTL8139_GW, sizeof(RTRTL8139_GW));
    string2ip_addr(&netmask,RTRTL8139_NETMASK, sizeof(RTRTL8139_NETMASK));
    
    ethernetif = netif_add(&ipaddr, &netmask, &gw, NULL, rt_rtl8139if_init, tcpip_input);
    
    rtl_printf("rt_rtl8139 interface added to netif list\n");
    
    if(!RT3COM905C_ISDEFAULTIF){
      if(RTRTL8139_ISDEFAULTIF) { 
	netif_set_default(ethernetif);
	rtl_printf("rt_rtl8139 set as default\n");
      }
    }else
      rtl_printf("Only one interface can be set as default");
  }
#endif //__RTETHERNETOPTS__

#ifdef __RTFIFOOPTS__

  string2ip_addr(&ipaddr,RTFIFO_IP, sizeof(RTFIFO_IP));
  string2ip_addr(&gw,RTFIFO_GW, sizeof(RTFIFO_GW));
  string2ip_addr(&netmask,RTFIFO_NETMASK, sizeof(RTFIFO_NETMASK));

  /* RTFifo interface is created and IP adress RTFIFO_IP is asigned to the RTFifo device */
  if(RTFIFO_ISDEFAULTIF){
    netif_set_default(netif_add(&ipaddr, &netmask, &gw,NULL, rtfifoif_init, tcpip_input));
    rtl_printf("RTFIFO interface added to netif list\n");
    rtl_printf("RTFIFO set as default\n");
  }else{
    netif_add(&ipaddr, &netmask, &gw,NULL, rtfifoif_init,
	      tcpip_input);
    rtl_printf("RTFIFO interface added to netif list\n");
  }

#endif /* ___RTFIFOOPTS__ */

  IP4_ADDR(&gw, 127,0,0,1);
  IP4_ADDR(&ipaddr, 127,0,0,1);
  IP4_ADDR(&netmask, 255,0,0,0);
  netif_add(&ipaddr, &netmask, &gw,NULL, loopif_init,
	    tcpip_input);

  /* Block for ever. */
  sem = sys_sem_new(0);
  sys_sem_wait(sem);
}


/*-----------------------------------------------------------------------------------*/
int init_module(void){
  printk("\n\nRTLinux TCP/IP stack inserted. linux thread id: 0x%x\n\n ",(unsigned int) pthread_self());

  sys_init();
   
  //This thread must be standalone, because it will create other threads
  sys_thread_new((void *)(main_thread), NULL, 0);


#ifdef __RTFIFOOPTS__
  /* Better be sure that RTFifos are not being used */
  rtf_destroy(0);
  rtf_destroy(1);
  
  /*
     RTFifos are uni-directional, for bi-directional data exchange two RTFifos must be
     created (/dev/rtf0 and /dev/rtf1).
  */
  rtf_create(0, 4000);
  rtf_create(1, 4000);
  
#endif /* ___RTFIFOOPTS__ */

  return 0;
}

/*-----------------------------------------------------------------------------------*/
void cleanup_module(void){

  sys_arch_close(); 

#ifdef __RTFIFOOPTS__
  /* RTFifos must be destroyed */
  rtf_destroy(0);
  rtf_destroy(1);
#endif /* ___RTFIFOOPTS__ */

#ifdef __RTETHERNETOPTS__
  if(RT3COM905C_ISAVAILABLE)
    rt_3c905cif_close();

  if(RTRTL8139_ISAVAILABLE)
    rt_rtl8139if_close();
#endif

  printk("\n\nRTLinux TCP/IP stack removed.\n\n");
}
