/*
 * Copyright (c) 2001-2003 Swedish Institute of Computer Science.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 */

#include "lwip/ip_addr.h"
#include "lwip/inet.h"
#include <string.h>


#define htons HTONS
#define htonl HTONL

struct ip_addr ip_addr_any = { 0x00000000UL };
struct ip_addr ip_addr_broadcast = { 0xffffffffUL };

/*-----------------------------------------------------------------------------------*/
int get_int_from_char(char c){

  /* 0-9 */
  if((c >= 48) && (c <= 57)) return (c-48);
  return -1;
}

/*-----------------------------------------------------------------------------------*/
void string2ip_addr(struct ip_addr *ip, char *name, int size){
  int i, index=0, sum[4]={0,0,0,0}, ciphers[3]={0,0,0},cont=0;

  if(name != NULL){
    for(i=0; i<3; i++){
      do{
        ciphers[cont]=get_int_from_char(name[index]);
        index++;
        cont++;
      }while(name[index] != 0x2e); //0x2e is the hex ascii code for '.'
      if(cont==3)
        sum[i] = ciphers[0]*100 + ciphers[1]*10 + ciphers[2];
      else if(cont==2)
        sum[i] = ciphers[0]*10 + ciphers[1];
      else
        sum[i] = ciphers[0];
      index++;
      ciphers[0] = 0; ciphers[1]= 0; ciphers[2] = 0; cont = 0;
    }

    while(name[index]!='\0'){
      ciphers[cont]= get_int_from_char(name[index]);
      index++;
      cont++;
    }
    if(cont==3)
      sum[3] = ciphers[0]*100 + ciphers[1]*10 + ciphers[2];
    else if(cont==2)
      sum[3] = ciphers[0]*10 + ciphers[1];
    else
      sum[3] = ciphers[0];

    IP4_ADDR(ip, sum[0],sum[1],sum[2],sum[3]);
  }
}
/*-----------------------------------------------------------------------------------*/
